#v1.0
#v0.9 - All research graph via menu & mouse click
#v0.8 - Candlestick graphs
#v0.7 - Base version with all graphs and bug fixes
#v0.6
import pandas as pd
from pandas import DataFrame
from alpha_vantage.timeseries import TimeSeries
from alpha_vantage.techindicators import TechIndicators

class PrepareTestData():
    def __init__(self, argFolder='./scriptdata', argOutputSize='compact'):
        super().__init__()
        self.folder = argFolder + '/'
        self.outputsize = argOutputSize.lower()
    
    def loadDaily(self, argScript):
        try:
            if(self.outputsize == 'compact'):
                filename=self.folder + 'daily_compact_'+argScript+'.csv'
            else:
                filename=self.folder + 'daily_full_'+argScript+'.csv'
            csvdf = pd.read_csv(filename)
            csvdf=csvdf.rename(columns={'open':'1. open', 'high':'2. high', 'low':'3. low', 'close':'4. close', 'volume': '5. volume'})
            convert_type={'1. open':float, '2. high':float, '3. low':float, '4. close':float, '5. volume':float}
            csvdf = csvdf.astype(convert_type)

            csvdf.set_index('timestamp', inplace=True)

            csvdf.index = pd.to_datetime(csvdf.index)
            csvdf.index.names = ['date']
        except Exception as e:
            csvdf = DataFrame()
        
        return csvdf

    def loadIntra(self, argScript):
        try:
            if(self.outputsize == 'compact'):
                filename=self.folder + 'intraday_5min_compact_'+argScript+'.csv'
            else:
                filename=self.folder + 'intraday_5min_full_'+argScript+'.csv'

            csvdf = pd.read_csv(filename)
            csvdf=csvdf.rename(columns={'open':'1. open', 'high':'2. high', 'low':'3. low', 'close':'4. close', 'volume': '5. volume'})
            convert_type={'1. open':float, '2. high':float, '3. low':float, '4. close':float, '5. volume':float}
            csvdf = csvdf.astype(convert_type)

            csvdf.set_index('timestamp', inplace=True)

            csvdf.index = pd.to_datetime(csvdf.index)
            csvdf.index.names = ['date']
        except Exception as e:
            csvdf = DataFrame()
        
        return csvdf

    def loadSMA(self, argScript='', argPeriod=20):
        try:
            #if(argPeriod == 0):
            #    csvdf = pd.read_csv(self.folder + 'SMA_'+argScript+'.csv')
            #else:
            csvdf = pd.read_csv(self.folder + 'SMA_'+str(argPeriod)+ '_'+argScript+'.csv')

            convert_type={'SMA':float}
            csvdf = csvdf.astype(convert_type)

            csvdf.set_index('time', inplace=True)
            csvdf.index = pd.to_datetime(csvdf.index)
            csvdf.index.names = ['date']

            #ti = TechIndicators('XXXX', output_format='pandas')
            #padf, pameta = ti.get_sma(argScript)

        except Exception as e:
            csvdf = DataFrame()
        
        return csvdf

    def loadEMA(self, argScript):
        try:
            csvdf = pd.read_csv(self.folder + 'EMA_'+argScript+'.csv')
            convert_type={'EMA':float}
            csvdf = csvdf.astype(convert_type)

            csvdf.set_index('time', inplace=True)
            csvdf.index = pd.to_datetime(csvdf.index)
            csvdf.index.names = ['date']

            #ti = TechIndicators('XXXX', output_format='pandas')
            #padf, pameta = ti.get_ema(argScript)

        except Exception as e:
            csvdf = DataFrame()
        return csvdf

    def loadVWMP(self, argScript):
        try:
            csvdf = pd.read_csv(self.folder + 'VWAP_'+argScript+'.csv')
            convert_type={'VWAP':float}
            csvdf = csvdf.astype(convert_type)

            csvdf.set_index('time', inplace=True)
            csvdf.index = pd.to_datetime(csvdf.index)
            csvdf.index.names = ['date']

            #ti = TechIndicators('XXXX', output_format='pandas')
            #padf, pameta = ti.get_ema(argScript)

        except Exception as e:
            csvdf = DataFrame()

        return csvdf

    def loadRSI(self, argScript):
        try:
            csvdf = pd.read_csv(self.folder + 'RSI_'+argScript+'.csv')
            convert_type={'RSI':float}
            csvdf = csvdf.astype(convert_type)

            csvdf.set_index('time', inplace=True)
            csvdf.index = pd.to_datetime(csvdf.index)
            csvdf.index.names = ['date']

            #ti = TechIndicators('XXXX', output_format='pandas')
            #padf, pameta = ti.get_ema(argScript)

        except Exception as e:
            csvdf = DataFrame()
        return csvdf

    def loadStochasticOscillator(self, argScript):
        try:
            csvdf = pd.read_csv(self.folder + 'STOCH_'+argScript+'.csv')
            convert_type={'SlowD':float, 'SlowK':float}
            csvdf = csvdf.astype(convert_type)

            csvdf.set_index('time', inplace=True)
            csvdf.index = pd.to_datetime(csvdf.index)
            csvdf.index.names = ['date']

            #ti = TechIndicators('XXXX', output_format='pandas')
            #padf, pameta = ti.get_ema(argScript)

        except Exception as e:
            csvdf = DataFrame()
        return csvdf

    def loadMACD(self, argScript):
        try:
            csvdf = pd.read_csv(self.folder + 'MACD_'+argScript+'.csv')
            convert_type={'MACD':float, 'MACD_Hist':float, 'MACD_Signal':float}
            csvdf = csvdf.astype(convert_type)

            csvdf.set_index('time', inplace=True)
            csvdf.index = pd.to_datetime(csvdf.index)
            csvdf.index.names = ['date']

            #ti = TechIndicators('XXXX', output_format='pandas')
            #padf, pameta = ti.get_ema(argScript)

        except Exception as e:
            csvdf = DataFrame()
        return csvdf

    def loadAROON(self, argScript):
        try:
            csvdf = pd.read_csv(self.folder + 'AROON_'+argScript+'.csv')
            convert_type={'Aroon Down':float, 'Aroon Up':float}
            csvdf = csvdf.astype(convert_type)

            csvdf.set_index('time', inplace=True)
            csvdf.index = pd.to_datetime(csvdf.index)
            csvdf.index.names = ['date']

            #ti = TechIndicators('XXXX', output_format='pandas')
            #padf, pameta = ti.get_ema(argScript)

        except Exception as e:
            csvdf = DataFrame()
        return csvdf

    def loadBBands(self, argScript):
        try:
            csvdf = pd.read_csv(self.folder + 'BBANDS_'+argScript+'.csv')
            convert_type={'Real Lower Band':float, 'Real Middle Band':float, 'Real Upper Band':float}
            csvdf = csvdf.astype(convert_type)

            csvdf.set_index('time', inplace=True)
            csvdf.index = pd.to_datetime(csvdf.index)
            csvdf.index.names = ['date']

            #ti = TechIndicators('XXXX', output_format='pandas')
            #padf, pameta = ti.get_ema(argScript)

        except Exception as e:
            csvdf = DataFrame()
        return csvdf

    def loadADX(self, argScript):
        try:
            csvdf = pd.read_csv(self.folder + 'ADX_'+argScript+'.csv')
            convert_type={'ADX':float}
            csvdf = csvdf.astype(convert_type)

            csvdf.set_index('time', inplace=True)
            csvdf.index = pd.to_datetime(csvdf.index)
            csvdf.index.names = ['date']

            #ti = TechIndicators('XXXX', output_format='pandas')
            #padf, pameta = ti.get_ema(argScript)

        except Exception as e:
            csvdf = DataFrame()
        return csvdf

    def GetQuoteEndPoint(self, argScript):
        try:
            csvdf = pd.read_csv(self.folder + 'global_quote_'+argScript+'.csv')

            csvdf=csvdf.rename(columns={'symbol':'01. symbol', 'open':'02. open', 'high':'03. high', 'low':'04. low', 'price': '05. price', 'volume':'06. volume', 'latestDay':'07. latest trading day', 'previousClose':'08. previous close', 'change': '09. change', 'changePercent':'10. change percent'})
            convert_type = {'01. symbol': object, '02. open':object, '03. high':object, '04. low':object, '05. price':object, '06. volume':object, '07. latest trading day':object, '08. previous close':object, '09. change':object, '10. change percent':object}
            csvdf = csvdf.astype(convert_type)
            csvdf.index = ['Global Quote']

        except Exception as e:
            csvdf = DataFrame()
        return csvdf


    def loadDailyTrial(self, argScript):

        """stemp = {"2020-03-20": ["1640.0000","1778.0000","1536.0500","1753.9000","327948"], 
                "2020-03-19": ["1640.0000","1778.0000","1536.0500","1753.9000","327948"]}"""
        
        csvdf = pd.read_csv(self.folder + 'daily_'+argScript+'.csv')

        csvdf=csvdf.rename(columns={'open':'1. open', 'high':'2. high', 'low':'3. low', 'close':'4. close', 'volume': '5. volume'})
        convert_type={'1. open':float, '2. high':float, '3. low':float, '4. close':float, '5. volume':float}
        csvdf = csvdf.astype(convert_type)

        csvdf.set_index('timestamp', inplace=True)

        csvdf.index = pd.to_datetime(csvdf.index)
        csvdf.index.names = ['date']


        filename = self.folder + 'daily_'+argScript+'.data'
        
        stemp = ''
        fhandle = open(filename, 'r')
        for line in fhandle:
            line = line.rstrip('\n')
            line = line.lstrip()
            stemp = stemp + line

        stemp=eval(stemp)
        dailydf = DataFrame.from_dict(stemp, orient='index', columns=['1. open', '2. high', '3. low', '4. close', '5. volume'])
        
        convert_type={'1. open':float, '2. high':float, '3. low':float, '4. close':float, '5. volume':float}
        dailydf = dailydf.astype(convert_type)

        dailydf.index = pd.to_datetime(dailydf.index)
        dailydf.index.names = ['date']

        ts = TimeSeries('XXXX', output_format='pandas')
        padf, pameta = ts.get_daily('HDFC.BSE')

if __name__ == "__main__":
    obj = PrepareTestData()
    obj.loadDaily('HDFC.BSE')
    input()



#v1.0
import os
import requests
import time
from tkinter import *
from tkinter import ttk
from tkinter import messagebox as msgbx

import pandas as pd
from pandas import DataFrame

class classDownloadData(Toplevel):
    def __init__(self, master=None, argKey= 'XXXX', argFolder='./ScriptData', argMFdownloadFromDays=90, **kw):
        super().__init__(master=master, **kw)

        self.wm_title("Download Data")
        self.wm_protocol("WM_DELETE_WINDOW", self.OnClose)
        #XXXX
        #UV6KQA6735QZKBTV
        self.key = 'UV6KQA6735QZKBTV'
        self.folder = argFolder
        self.outputsize='compact'
        self.configure(padx=5, pady=10)
        self.bisfulldownload = True
        self.mb = ttk.Menubutton (self, text="Select items to download")
        self.mb.grid()
        self.mb.menu  =  Menu(self.mb, tearoff=0)
        self.mb["menu"]  = self.mb.menu

        self.Item0 = IntVar()
        self.Item1 = IntVar()
        self.Item2 = IntVar()
        self.Item3 = IntVar()
        self.Item4 = IntVar()
        self.Item5 = IntVar()
        self.Item6 = IntVar()
        self.Item7 = IntVar()
        self.Item8 = IntVar()
        self.Item9 = IntVar()
        self.Item10 = IntVar()
        self.Item11 = IntVar()

        self.mb.menu.add_checkbutton ( label="Daily price", variable=self.Item0)
        self.mb.menu.add_checkbutton ( label="Intraday", variable=self.Item1)
        self.mb.menu.add_checkbutton ( label="Simple moving avg", variable=self.Item2)
        self.mb.menu.add_checkbutton ( label="Volume weighted avg price", variable=self.Item3)
        self.mb.menu.add_checkbutton ( label="Relative strength index", variable=self.Item4)
        self.mb.menu.add_checkbutton ( label="Avg directional movement index", variable=self.Item5)
        self.mb.menu.add_checkbutton ( label="Stochastic oscillator", variable=self.Item6)
        self.mb.menu.add_checkbutton ( label="Moving average convergence/divergence", variable=self.Item7)
        self.mb.menu.add_checkbutton ( label="Aroon", variable=self.Item8)
        self.mb.menu.add_checkbutton ( label="Bollinger bands", variable=self.Item9)
        self.mb.menu.add_checkbutton ( label="EMA", variable=self.Item10)
        self.mb.menu.add_checkbutton ( label="Quote end point", variable=self.Item11)

        self.btn_download_selected = ttk.Button(self, text='Download Selected', command=self.btnDownloadSelected)

        self.btn_download_full = ttk.Button(self, text='Download All (outputsize=full)', command=self.btnDownloadFull)
        self.btn_download_compact = ttk.Button(self, text="Download All (outputsize=compact)", command=self.btnDownloadCompact)
        self.btn_close = ttk.Button(self, text="Close", command=self.OnClose)
        self.progressbar = ttk.Progressbar(self, length=130, orient= HORIZONTAL, mode='determinate')

        self.mb.grid_configure(row=0, column=1, padx=5, pady=5)
        self.btn_download_selected.grid_configure(row=0, column=2, padx=5, pady=5)

        self.btn_download_full.grid_configure(row=1, column=1, padx=5, pady=5)
        self.btn_download_compact.grid_configure(row=1, column=2, padx=5, pady=5)
        self.progressbar.grid_configure(row=1, column=3, padx=5, pady=5)
        self.btn_close.grid_configure(row=1, column=4, padx=5, pady=5)

       
        self.grid_columnconfigure(0, weight=1)
        self.grid_columnconfigure(1, weight=1)
        self.grid_columnconfigure(2, weight=1)
        self.grid_columnconfigure(3, weight=1)
        self.grid_columnconfigure(4, weight=1)
        self.grid_rowconfigure(0, weight=1)
        self.grid_rowconfigure(1, weight=1)

    def OnClose(self):
        self.destroy()

    def downloadQuoteEndPoint(self, arglistscriptname):
        try:
            for script in arglistscriptname:
                url = 'https://www.alphavantage.co/query?function=GLOBAL_QUOTE&symbol={}&apikey={}&datatype=csv'.format(script, self.key)
                filename = 'global_quote_{}.csv'.format(script)
                response = requests.get(url)
                with open(os.path.join(self.folder, filename), 'wb') as f:
                    f.write(response.content)
                    f.close()
                time.sleep(60)
                print(filename)
        except Exception as e:
            msgbx.showerror('Error in downloadQuoteEndPoint', str(e))
            """self.progressbar['value'] = 10
            self.progressbar.update()
            self.update_idletasks()"""

    def downloadIntra(self, arglistscriptname):
        try:
            for script in arglistscriptname:
                url = 'https://www.alphavantage.co/query?function=TIME_SERIES_INTRADAY&symbol={}&interval=5min&apikey={}&outputsize={}&datatype=csv'.format(script, self.key, self.outputsize)
                filename = 'intraday_5min_{}_{}.csv'.format(self.outputsize, script)
                response = requests.get(url)
                with open(os.path.join(self.folder, filename), 'wb') as f:
                    f.write(response.content)
                    f.close()
                time.sleep(60)
                print(filename)
        except Exception as e:
            print(str(e))

            """self.progressbar['value'] = 10
            self.progressbar.update()
            self.update_idletasks()"""

    def downloadDaily(self, arglistscriptname):
        try:
            for script in arglistscriptname:
                url = 'https://www.alphavantage.co/query?function=TIME_SERIES_DAILY&symbol={}&apikey={}&outputsize={}&datatype=csv'.format(script, self.key, self.outputsize)
                filename = 'daily_{}_{}.csv'.format(self.outputsize, script)
                
                response = requests.get(url)
                with open(os.path.join(self.folder, filename), 'wb') as f:
                    f.write(response.content)
                    f.close()
                time.sleep(60)
                print(filename)
        except Exception as e:
            print(str(e))

            """self.progressbar['value'] = 10
            self.progressbar.update()
            self.update_idletasks()"""

    def downloadSMA(self, arglistscriptname):
        try:
            for script in arglistscriptname:
                url = 'https://www.alphavantage.co/query?function=SMA&symbol={}&interval=daily&time_period=20&series_type=close&apikey={}&datatype=csv'.format(script, self.key)
                filename = 'SMA_20_{}.csv'.format(script)
                response = requests.get(url)
                with open(os.path.join(self.folder, filename), 'wb') as f:
                    f.write(response.content)
                    f.close()
                time.sleep(60)
                print(filename)
        except Exception as e:
            print(str(e))

            """self.progressbar['value'] = 10
            self.progressbar.update()
            self.update_idletasks()"""
        try:
            for script in arglistscriptname:
                url = 'https://www.alphavantage.co/query?function=SMA&symbol={}&interval=daily&time_period=10&series_type=close&apikey={}&datatype=csv'.format(script, self.key)
                filename = 'SMA_10_{}.csv'.format(script)
                response = requests.get(url)
                with open(os.path.join(self.folder, filename), 'wb') as f:
                    f.write(response.content)
                    f.close()
                time.sleep(60)
                print(filename)
        except Exception as e:
            print(str(e))

            """self.progressbar['value'] = 10
            self.progressbar.update()
            self.update_idletasks()"""

    def downloadEMA(self, arglistscriptname):
        try:
            for script in arglistscriptname:
                url = 'https://www.alphavantage.co/query?function=EMA&symbol={}&interval=daily&time_period=20&series_type=close&apikey={}&datatype=csv'.format(script, self.key)
                filename = 'EMA_{}.csv'.format(script)
                response = requests.get(url)
                with open(os.path.join(self.folder, filename), 'wb') as f:
                    f.write(response.content)
                    f.close()
                time.sleep(60)
                print(filename)
        except Exception as e:
            print(str(e))

            """self.progressbar['value'] = 10
            self.progressbar.update()
            self.update_idletasks()"""

    def downloadVWAP(self, arglistscriptname):
        try:
            for script in arglistscriptname:
                url = 'https://www.alphavantage.co/query?function=VWAP&symbol={}&interval=5min&apikey={}&datatype=csv'.format(script, self.key)
                filename = 'VWAP_{}.csv'.format(script)
                response = requests.get(url)
                with open(os.path.join(self.folder, filename), 'wb') as f:
                    f.write(response.content)
                    f.close()
                time.sleep(60)
                print(filename)
        except Exception as e:
            print(str(e))

            """self.progressbar['value'] = 10
            self.progressbar.update()
            self.update_idletasks()"""

    def downloadRSI(self, arglistscriptname):
        try:
            for script in arglistscriptname:
                url = 'https://www.alphavantage.co/query?function=RSI&symbol={}&interval=daily&time_period=20&series_type=close&apikey={}&datatype=csv'.format(script, self.key)
                filename = 'RSI_{}.csv'.format(script)
                response = requests.get(url)
                with open(os.path.join(self.folder, filename), 'wb') as f:
                    f.write(response.content)
                    f.close()
                time.sleep(60)
                print(filename)
        except Exception as e:
            print(str(e))

            """self.progressbar['value'] = 10
            self.progressbar.update()
            self.update_idletasks()"""

    def downloadSTOCH(self, arglistscriptname):
        try:
            for script in arglistscriptname:
                url = 'https://www.alphavantage.co/query?function=STOCH&symbol={}&interval=daily&apikey={}&datatype=csv'.format(script, self.key)
                filename = 'STOCH_{}.csv'.format(script)
                response = requests.get(url)
                with open(os.path.join(self.folder, filename), 'wb') as f:
                    f.write(response.content)
                    f.close()
                time.sleep(60)
                print(filename)
        except Exception as e:
            print(str(e))

            """self.progressbar['value'] = 10
            self.progressbar.update()
            self.update_idletasks()"""

    def downloadMACD(self, arglistscriptname):
        try:
            for script in arglistscriptname:
                url = 'https://www.alphavantage.co/query?function=MACD&symbol={}&interval=daily&series_type=close&apikey={}&datatype=csv'.format(script, self.key)
                filename = 'MACD_{}.csv'.format(script)
                response = requests.get(url)
                with open(os.path.join(self.folder, filename), 'wb') as f:
                    f.write(response.content)
                    f.close()
                time.sleep(60)
                print(filename)
        except Exception as e:
            print(str(e))

            """self.progressbar['value'] = 10
            self.progressbar.update()
            self.update_idletasks()"""

    def downloadAROON(self, arglistscriptname):
        try:
            for script in arglistscriptname:
                url = 'https://www.alphavantage.co/query?function=AROON&symbol={}&interval=daily&time_period=20&apikey={}&datatype=csv'.format(script, self.key)
                filename = 'AROON_{}.csv'.format(script)
                response = requests.get(url)
                with open(os.path.join(self.folder, filename), 'wb') as f:
                    f.write(response.content)
                    f.close()
                time.sleep(60)
                print(filename)
        except Exception as e:
            print(str(e))

            """self.progressbar['value'] = 10
            self.progressbar.update()
            self.update_idletasks()"""

    def downloadBBANDS(self, arglistscriptname):
        try:
            for script in arglistscriptname:
                url = 'https://www.alphavantage.co/query?function=BBANDS&symbol={}&interval=daily&time_period=20&series_type=close&nbdevup=2&nbdevdn=2&apikey={}&datatype=csv'.format(script, self.key)
                filename = 'BBANDS_{}.csv'.format(script)
                response = requests.get(url)
                with open(os.path.join(self.folder, filename), 'wb') as f:
                    f.write(response.content)
                    f.close()
                time.sleep(60)
                print(filename)
        except Exception as e:
            print(str(e))

            """self.progressbar['value'] = 10
            self.progressbar.update()
            self.update_idletasks()"""

    def downloadADX(self, arglistscriptname):
        try:
            for script in arglistscriptname:
                url = 'https://www.alphavantage.co/query?function=ADX&symbol={}&interval=daily&time_period=20&apikey={}&datatype=csv'.format(script, self.key)
                filename = 'ADX_{}.csv'.format(script)
                response = requests.get(url)
                with open(os.path.join(self.folder, filename), 'wb') as f:
                    f.write(response.content)
                    f.close()
                time.sleep(60)
                print(filename)
        except Exception as e:
            print(str(e))

            """self.progressbar['value'] = 10
            self.progressbar.update()
            self.update_idletasks()"""

    def btnDownloadCompact(self):
        self.outputsize = 'compact'
        self.downloadIntra(['HDFC.BSE', 'LT.BSE', 'BAJFINANCE.BSE', 'AAPL'])
        self.downloadDaily(['HDFC.BSE', 'LT.BSE', 'BAJFINANCE.BSE', 'AAPL'])
        self.progressbar['value'] = 100
        self.progressbar.update_idletasks() 

    def btnDownloadFull(self):
        self.outputsize = 'full'
        self.bisfulldownload = True
        self.btnDownload()


    def btnDownload(self):
        self.progressbar['value'] = 20
        self.progressbar.update_idletasks() 
        if(self.bisfulldownload or self.Item0.get()):
            self.downloadDaily(['HDFC.BSE', 'LT.BSE', 'BAJFINANCE.BSE', 'AAPL'])
            self.progressbar['value'] = 30
            self.progressbar.update_idletasks() 
            time.sleep(60)

        if(self.bisfulldownload or self.Item1.get()):
            self.downloadIntra(['HDFC.BSE', 'LT.BSE', 'BAJFINANCE.BSE', 'AAPL'])
            self.progressbar['value'] = 40
            self.progressbar.update_idletasks() 
            time.sleep(60)
        
        if(self.bisfulldownload or self.Item2.get()):
            self.downloadSMA(['HDFC.BSE', 'LT.BSE', 'BAJFINANCE.BSE', 'AAPL'])
            self.progressbar['value'] = 50
            self.progressbar.update_idletasks() 
            time.sleep(60)
        
        if(self.bisfulldownload or self.Item3.get()):
            self.downloadVWAP(['HDFC.BSE', 'LT.BSE', 'BAJFINANCE.BSE', 'AAPL'])
            self.progressbar['value'] = 70
            self.progressbar.update_idletasks() 
            time.sleep(60)
       
        if(self.bisfulldownload or self.Item4.get()):
            self.downloadRSI(['HDFC.BSE', 'LT.BSE', 'BAJFINANCE.BSE', 'AAPL'])
            self.progressbar['value'] = 80
            self.progressbar.update_idletasks() 
            time.sleep(60)

        if(self.bisfulldownload or self.Item5.get()):
            self.downloadADX(['HDFC.BSE', 'LT.BSE', 'BAJFINANCE.BSE', 'AAPL'])
            self.progressbar['value'] = 10
            self.progressbar.update_idletasks() 
            time.sleep(60)

        if(self.bisfulldownload or self.Item6.get()):
            self.downloadSTOCH(['HDFC.BSE', 'LT.BSE', 'BAJFINANCE.BSE', 'AAPL'])
            self.progressbar['value'] = 90
            self.progressbar.update_idletasks() 
            time.sleep(60)
        
        if(self.bisfulldownload or self.Item7.get()):
            self.downloadMACD(['HDFC.BSE', 'LT.BSE', 'BAJFINANCE.BSE', 'AAPL'])
            self.progressbar['value'] = 95
            self.progressbar.update_idletasks() 
            time.sleep(60)

        if(self.bisfulldownload or self.Item8.get()):
            self.downloadAROON(['HDFC.BSE', 'LT.BSE', 'BAJFINANCE.BSE', 'AAPL'])
            self.progressbar['value'] = 96
            self.progressbar.update_idletasks() 
            time.sleep(60)

        if(self.bisfulldownload or self.Item9.get()):
            self.downloadBBANDS(['HDFC.BSE', 'LT.BSE', 'BAJFINANCE.BSE', 'AAPL'])
            self.progressbar['value'] = 97
            self.progressbar.update_idletasks() 
            time.sleep(60)

        if(self.bisfulldownload or self.Item10.get()):
            self.downloadEMA(['HDFC.BSE', 'LT.BSE', 'BAJFINANCE.BSE', 'AAPL'])
            self.progressbar['value'] = 60
            self.progressbar.update_idletasks() 
            time.sleep(60)

        if(self.bisfulldownload or self.Item11.get()):
            self.downloadQuoteEndPoint(['HDFC.BSE', 'LT.BSE', 'BAJFINANCE.BSE', 'AAPL'])
            self.progressbar['value'] = 100
            self.progressbar.update_idletasks() 


    def btnDownloadMF(self):
        #http://portal.amfiindia.com/DownloadNAVHistoryReport_Po.aspx?frmdt=01-Apr-2020
        #http://portal.amfiindia.com/DownloadNAVHistoryReport_Po.aspx?mf=27&frmdt=01-Apr-2020&todt=11-Apr-2020
        url = 'http://portal.amfiindia.com/DownloadNAVHistoryReport_Po.aspx?mf=27&frmdt=01-Apr-2020&todt=11-Apr-2020'
        filename = 'franklin.csv'
        response = requests.get(url)
        with open(os.path.join(self.folder, filename), 'wb') as f:
            f.write(response.content)
            f.close()
        
        frandf = pd.read_csv(filepath_or_buffer=os.path.join(self.folder, filename), 
            delimiter=';', header=0)
        print(frandf)
        dftest = frandf[frandf['Date'].notnull()]
        print(dftest)
        input()

    def btnDownloadSelected(self):
        self.outputsize = 'full'
        self.bisfulldownload = False
        self.btnDownload()



if __name__ == "__main__":
    obj1 = classDownloadData()
    input()



#v1.0
#v0.9 - All research graph via menu & mouse click
#v0.8 - Candlestick graphs
#v0.7 - Base version with all graphs and bug fixes
#v0.6
#0.5
#v0.4
from tkinter import *
from tkinter import ttk
from tkinter import messagebox as msgbx
from tkinter.filedialog import *
from datetime import date
from tkcalendar import Calendar, DateEntry

class classAddKey(Toplevel):
    def __init__(self, master=None):
        Toplevel.__init__(self, master=master)
        
        self.key = ''
        self.datadir = ''        
        self.wm_title("Add Key")

        self.wm_protocol("WM_DELETE_WINDOW", self.OnClose)

        self.configure(padx=5, pady=10)
        self.iscancel = False

        self.frame1 = ttk.Frame(self, borderwidth=5, relief="sunken") #, width=200, height=100)

        # Now create purchase price entry
        self.key_label = ttk.Label(self.frame1, text='Enter your key: ')
        self.key_text = StringVar(value='XXXX')
        self.key_entry = ttk.Entry(self.frame1, textvariable=self.key_text, width=40)

        self.datafolder_label = ttk.Label(self.frame1, text='Folder for source data: ')
        self.datafolder_text = StringVar(value='')
        self.datafolder_entry = ttk.Entry(self.frame1, textvariable=self.datafolder_text, width=40, state='read')
        self.btn_add_datafolder = ttk.Button(self.frame1, text="Browse & Select Folder", command=self.btnDataFolder)

        self.btn_ok = ttk.Button(self.frame1, text="Ok", command=self.btnOk)
        self.btn_cancel = ttk.Button(self.frame1, text="Cancel", command=self.btnCancel)

        #put widgets on grid_configure
        self.frame1.grid_configure(row=0, column=0, sticky=(N, S, E, W), padx=5, pady=5)
        self.key_label.grid_configure(row=0, column=0, sticky=(N, S, E, W))
        self.key_entry.grid_configure(row=0, column=1, sticky=(N, S, E, W))

        self.datafolder_label.grid_configure(row=1, column=0, sticky=(N, S, E, W))
        self.datafolder_entry.grid_configure(row=1, column=1, sticky=(N, S, E, W))
        self.btn_add_datafolder.grid_configure(row=1, column=2, padx=5, pady=5, sticky=(N, S, E, W))

        self.btn_ok.grid_configure(row=2, column=1, padx=5, pady=5, sticky=(E))#, sticky=(N, S, E, W))
        self.btn_cancel.grid_configure(row=2, column=2, padx=5, pady=5)#, sticky=(N, S, E, W))

    def OnClose(self):
        self.destroy()

           
    def btnOk(self):
        self.iscancel = False
        self.destroy()

    def btnDataFolder(self):
        self.datafolder_text.set(askdirectory(initialdir = "./scriptdata", title = "Select source folder for data files"))
        self.btn_add_datafolder.focus_force()

    def btnCancel(self):
        self.iscancel = True
        self.destroy()

    def show(self):
        self.wm_deiconify()
        self.key_entry.focus_force()
        self.wait_window()
        if(self.iscancel == False):
            return {'key':self.key_text.get(), 'folder':self.datafolder_text.get(),}

        return {}

if __name__ == "__main__":
    obj = classAddKey()
    key = obj.show()
    print(key)



#v1.0
#v0.9 - All research graph via menu & mouse click
#v0.8 - Candlestick graphs
#v0.7 - Base version with all graphs and bug fixes, added code to github desktop
#v0.6
#v0.5
#v0.4 - Features as below
# 1. File->Save current scripts in tree as portfolio  to a file
# 2. File->Open existing portfolio file and render data in tree
# 3. Manage Portfolio->Add script to tree via menu
# 4. Manage Portfolio->Refresh selected script from tree. 
#       This takes current market price and updates the current value for that script
# 5. Manage portfolio->Delete selected script from portfolio. 
#       Entire data is deleted from Tree, but the file is not updated
# 6. Analyze script->Get Quote. Search script (type first few chars 
#       & enter or click search button). Click Get Quote to get current price.
#       You can select specific indicator to see the performance graph of current script
#       You can use Add script button to add the script in Tree
# 7. Analyze Script->Show historical proce series of selected script. Shows close price graph
#       Note: this should be move to right click menu
# 8. Analyze Script->Compare Price Vs SMA. Currently shows popup graph
#       Note: 7 & 8 needs to be merged and the graph needs to be shown in main window on
#               right click menu
# 9. Help-> Test Mode (On/Off). Toggle the test mode. In Test mode we use file to 
#       load specific script data
# 10.Mouse right click->Delete. Deletes the currently selected portfolio entry from tree only.
#       The data is not saved to file
# 11. Mouse right click->Modify selected script from tree. You can change the quantity,
#       rate, commission etc. Based on the values the cost of investment will be updated
#       This will also take current market price and update all current value field in Tree
# 5. Mouse right click->Performance. Shows current value for total holding, shows other
#       comparison graph and return graph as well
#       Note: this needs to be moved in main window

from tkinter import *
from tkinter import ttk
from tkinter import scrolledtext as tkst
from tkinter import messagebox as msgbx
from tkinter.filedialog import *

from alpha_vantage.timeseries import TimeSeries
from alpha_vantage.techindicators import TechIndicators
import pandas as pd
from pandas import DataFrame
from matplotlib.pyplot import Figure
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from matplotlib import interactive
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2Tk
import warnings
from datetime import date

from scripttree import *
from addnewmodifyscript import *
from backtestsma import *
from getquote import *
from testdata import *
from graphresearch import *
from downloaddata import *
from portfoliovaluation import *
from addaccesskey import *

class PortfolioManager:
    def __init__(self):
        super().__init__()

        self.bool_test = False
        self.output_counter = 0
        self.currentScript = ''
        #graph related variables
        self.LookbackYears = 1 #we will analyze last one year date from today
        self.graphctr = 1
        # Now set the stretch options so that the widget are seen properly when window is resized
        self.tickmark = '√' #Alt+251
        self.dictgraphmenu = ({'m1':(3, 3, 0)}, {'m2':(3, 3, 0)}, {'m3':(3, 3, 0)}, 
                              {'m4':(3, 3, 0)}, {'m5':(3, 3, 0)}, {'m6':(3, 3, 0)},
                              {'m7':(3, 3, 0)}, {'m8':(3, 3, 0)}, {'m9':(3, 3, 0)})
        #array to hold axes
        self.ax = [None, None, None, None, None, None, None, None, None]
        self.annotate_state = [None, None, None, None, None, None, None, None, None]
        #DF for current script
        self.dfDaily = DataFrame()
        self.dfSMA = DataFrame()
        self.dfIntra = DataFrame()
        self.dfVWMP = DataFrame()
        self.dfRSI = DataFrame()
        self.dfStoch = DataFrame()
        self.dfMACD = DataFrame()
        self.dfAROON = DataFrame()
        self.dfBBANDS = DataFrame()
        self.dfADX = DataFrame()

        self.event = None
        self.annotate_visible = False
        self.datafolderpath = './scriptdata'
        # Set Alpha Vantage key and create timeseriese and time indicator objects
        #self.key = 'XXXX'
        self.key = 'UV6KQA6735QZKBTV'
        # get your key from https://www.alphavantage.co/support/#api-key

        # ts = TimeSeries(key, output_format='json')
        self.ts = TimeSeries(self.key, output_format='pandas')
        self.ti = TechIndicators(self.key, output_format='pandas')

        # Now create tkinter root object and the frame object on which we will place the other widgets
        self.root = Tk()
        self.root.state('zoomed')    #this maximizes the app window
        self.root.title('Stock Analytics - Online mode')
        self.content = ttk.Frame(self.root, padding=(5, 5, 12, 0))
        self.content.grid(column=0, row=0, sticky=(N, S, E, W))

        self.root.wm_protocol("WM_DELETE_WINDOW", self.OnClose)

        # add main menu object
        self.menu= Menu(master=self.root)
        self.root.config(menu=self.menu)
        # add file menu
        self.file_menu=Menu(self.menu, tearoff=0)
        self.file_menu.add_command(label="New Portfolio", underline = 0, command=self.menuNewPortfolio)
        self.file_menu.add_command(label="Open Portfolio", underline = 0, command=self.menuOpenPortfolio)
        self.file_menu.add_command(label="Save Portfolio", underline = 0, command=self.menuSavePortfolio)
        self.file_menu.add_separator()
        self.file_menu.add_command(label="Exit", underline = 1, command=self.OnClose)
        self.menu.add_cascade(label='File', underline = 0, menu=self.file_menu)

        # add manage script menu
        self.script_menu=Menu(self.menu, tearoff=0)
        self.script_menu.add_command(label="Add New Script", underline = 0, command=self.menuAddScript)
        self.script_menu.add_command(label="Delete Selected Script from Portfolio", underline = 0, command=self.menuDeleteSelectedScriptFromPortfolio)
        self.script_menu.add_separator()
        self.script_menu.add_command(label="Get Quote", underline = 0, command=self.menuGetStockQuote)
        self.script_menu.add_separator()
        self.script_menu.add_command(label="Refresh Selected Script with Market Price", underline = 0, command=self.menuRefreshScriptData)
        self.script_menu.add_separator()
        self.script_menu.add_command(label="Portfolio performance", underline = 0, command=self.menuPortfolioPerformance)
        self.menu.add_cascade(label='Manage Portfolio', underline = 0, menu=self.script_menu)

        # research script menu
        self.research_menu=Menu(self.menu, tearoff=0)
        self.research_menu.add_command(label="Graphs", underline = 0, command=self.menuResearchGraphs)
        self.menu.add_cascade(label='Research', underline = 0, menu=self.research_menu)

        # add help menu
        self.help_menu=Menu(self.menu, tearoff=0)
        self.help_menu.add_command(label="Add Key & Data Folder", underline = 0, command=self.menuKeyDataFolder)
        self.help_menu.add_command(label="Mode (Online/Offline)", underline = 0, command=self.menuSetTestMode)
        self.help_menu.add_command(label="Download Data", underline = 0, command=self.menuDownloadData)
        self.menu.add_cascade(label='Help', underline = 0, menu=self.help_menu)

        # plot variable used to plot 9 standard graphs, enabled via right click menu
        self.f = Figure(figsize=(15,7), dpi=100, facecolor='w', edgecolor='k', tight_layout=True, linewidth=0.5)
        self.output_canvas=FigureCanvasTkAgg(self.f, master=self.content)
        self.toolbar_frame=Frame(master=self.root)
        self.toolbar = NavigationToolbar2Tk(self.output_canvas, self.toolbar_frame)
        self.output_canvas.mpl_connect('pick_event', self.OnPicker)

        self.xmouseposition = None
        self.ymouseposition = None

        self.output_tree = ScriptTreeView(self.content, self.ts, self.ti, self.f, self.bool_test, self.output_canvas, self.toolbar, self.datafolderpath, selectmode='browse')
        #self.output_tree.bind('<<TreeviewSelect>>', self.TreeViewSelectionChanged)

        self.popup_menu_righclick = Menu(self.menu, tearoff=0)
        self.popup_menu_righclick.add_command(label="Delete", underline = 0, command=self.menuDeleteSelectedScript)
        self.popup_menu_righclick.add_command(label="Modify", underline = 0, command=self.menuModifySelectedScript)
        self.popup_menu_righclick.add_separator()
        
        self.popup_menu_rightclick_performance = Menu(self.popup_menu_righclick, tearoff=0)
        self.popup_menu_rightclick_performance.add_command(label="Script-Show All", underline = 12, command=self.menuShowScriptPerformanceAllGraphs)
        self.popup_menu_rightclick_performance.add_command(label="Script-Show Performance", underline = 12, command=self.menuShowScriptPerformance)
        self.popup_menu_rightclick_performance.add_command(label="Script-Show Candle stick OHLC", underline = 12, command=self.menuShowScriptCandlestick)
        self.popup_menu_rightclick_performance.add_command(label="Script-Show Buy & Sell prediction", underline = 12, command=self.menuShowScriptBuySell)
        self.popup_menu_rightclick_performance.add_command(label="Script-Show Returns", underline = 12, command=self.menuShowScriptReturns)
        self.popup_menu_righclick.add_cascade(label='Script Performance Graphs', menu=self.popup_menu_rightclick_performance, underline=0)

        self.popup_menu_righclick.add_separator()

        self.POSrightclickmenuDailyVsSMA = BooleanVar(False)
        self.popup_menu_righclick.add_checkbutton(label="Daily closing Vs 20 SMA", onvalue=True, offvalue=False, variable=self.POSrightclickmenuDailyVsSMA, command=self.rightclickmenuDailyVsSMA)
        #self.POSrightclickmenuDailyVsSMA = 5

        self.POSrightclickmenuIntraDay = BooleanVar(False)
        self.popup_menu_righclick.add_checkbutton(label="Intra-day", onvalue=True, offvalue=False, variable=self.POSrightclickmenuIntraDay, command=self.rightclickmenuIntraDay)
        #self.POSrightclickmenuIntraDay = 6

        self.POSrightclickmenuVWAP=BooleanVar(False)
        self.popup_menu_righclick.add_checkbutton(label="Volume Weighted Avg Price", onvalue=True, offvalue=False, variable=self.POSrightclickmenuVWAP, command=self.rightclickmenuVWAP)
        #self.POSrightclickmenuVWAP=7

        self.POSrightclickmenuRSI= BooleanVar(False)
        self.popup_menu_righclick.add_checkbutton(label="RSI", onvalue=True, offvalue=False, variable=self.POSrightclickmenuRSI, command=self.rightclickmenuRSI)
        #self.POSrightclickmenuRSI=8

        self.POSrightclickmenuADX= BooleanVar(False)
        self.popup_menu_righclick.add_checkbutton(label="ADX", onvalue=True, offvalue=False, variable=self.POSrightclickmenuADX, command=self.rightclickmenuADX)
        #self.POSrightclickmenuADX=9
        
        self.POSrightclickmenuStochasticOscillator= BooleanVar(False)
        self.popup_menu_righclick.add_checkbutton(label="Stochastic Oscillator", onvalue=True, offvalue=False, variable=self.POSrightclickmenuStochasticOscillator, command=self.rightclickmenuStochasticOscillator)
        #self.POSrightclickmenuStochasticOscillator=10

        self.POSrightclickmenuMACD= BooleanVar(False)
        self.popup_menu_righclick.add_checkbutton(label="Moving Avg convergence/divergence", onvalue=True, offvalue=False, variable=self.POSrightclickmenuMACD, command=self.rightclickmenuMACD)
        #self.POSrightclickmenuMACD=11

        self.POSrightclickmenuAROON= BooleanVar(False)
        self.popup_menu_righclick.add_checkbutton(label="AROON", onvalue=True, offvalue=False, variable=self.POSrightclickmenuAROON, command=self.rightclickmenuAROON)
        #self.POSrightclickmenuAROON=12

        self.POSrightclickmenuBBands= BooleanVar(False)
        self.bbandmenu = self.popup_menu_righclick.add_checkbutton(label="Bollinger Bands", onvalue=True, offvalue=False, variable=self.POSrightclickmenuBBands, command=self.rightclickmenuBBands)
        #self.POSrightclickmenuBBands=13

        self.popup_menu_righclick.add_separator()

        self.popup_menu_righclick.add_command(label="Clear All Graphs", underline = 0, command=self.menuClearAllGraphs)
        #self.POSrightclickmenuBBands=13

        self.popup_ongraph = Menu(self.menu, tearoff=0)
        self.popup_ongraph.add_command(label="Detail Graph", command=self.OnDetailGraph)
        self.popup_ongraph.add_command(label="Analysis", command=self.OnAnalysis)        
        self.popup_ongraph.add_command(label="Clear Graph", command=self.OnClearGraph)        

        self.output_tree.bind('<Button-3>', self.OnRightClick)

        self.output_tree.grid(row=0, column=0, rowspan=1, columnspan=11, sticky=(N,E, W, S))
        
        self.output_tree.vert_scroll.grid(row=0, column=11, sticky=(N, E, S))
        self.output_tree.horiz_scroll.grid(row=1, column=0, columnspan=11, sticky=(N, E, W, S))

        self.output_canvas.get_tk_widget().grid(row=2, column=0, columnspan=11, sticky=(N, E, W))

        self.toolbar_frame.grid(row=3, column=0, columnspan=11, sticky=(N, E, W))
        self.toolbar.grid(row=0, column=0, sticky=(N, W))

        #initialize the 9 plot areas and set the visibility to false
        for i in range(9):
            self.ax[i] = self.f.add_subplot(3,3,i+1, visible=False)

        # handles returned by the mouse move and mouse click event on plot areas
        self.cid_leftclick = None
        self.cid_leftrelease = None
        self.cid_mouse_move = None

        self.root.columnconfigure(0, weight=1)
        self.root.rowconfigure(0, weight=1)
        self.content.columnconfigure(0, weight=1)

        mainloop()

    def OnClose(self):
        self.root.destroy()

    def resetExisting(self):
        #global output_counter
        # delete existing tree items
        self.currentScript = ''
        self.output_tree.delete(*self.output_tree.get_children())
        bShouldRestMenu = self.TreeViewSelectionChanged()
        if( bShouldRestMenu):
            self.resetMenuticktoFalse()
        self.f.clf()
        if(self.output_tree.output_counter > 0):
            self.output_tree.output_counter = 1

    """Method - menuAddScript
        Adds a new script to the tree view """
    def menuAddScript(self):
        dnewscript = dict()
        dnewscript = classAddNewModifyScript(master=self.root, argkey=self.key).show()
        # returns dictionary - {'Symbol': 'LT.BSE', 'Price': '1000', 'Date': '2020-02-22', 'Quantity': '10', 'Commission': '1', 'Cost': '10001.0'}
        if((dnewscript != None) and (len(dnewscript['Symbol']) >0)):
            stock_name = dnewscript['Symbol']
            listnewscript = list(dnewscript.items())
            #argHoldingIID="", argStockName=stock_name, argPriceDf=DataFrame()
            self.output_tree.get_stock_quote("", stock_name, DataFrame(), listnewscript[1][0] + '=' +listnewscript[1][1],
                                            listnewscript[2][0] + '=' + listnewscript[2][1],
                                            listnewscript[3][0] + '=' + listnewscript[3][1],
                                            listnewscript[4][0] + '=' + listnewscript[4][1],
                                            listnewscript[5][0] + '=' + listnewscript[5][1])
            #dnewscript['Price'], dnewscript['Date'], 
            #   dnewscript['Quantity'], dnewscript['Commission'], dnewscript['Cost'])


    """ menuDeleteSelectedScriptFromPortfolio """
    def menuDeleteSelectedScriptFromPortfolio(self):
        item = self.output_tree.get_parent_item()
        if(len(item) > 0):
            try:
                if(msgbx.askyesno('Delete script from portfolio', 'Are you sure you want to delete entire script data?: '+ item + '?')):
                    self.output_tree.delete(item)
                    self.output_tree.update()
                    msgbx.showinfo('Delete Script from portfolio', "Selected script deleted successfully from portfolio. Please make sure to save portfolio!")
            except Exception as e:
                msgbx.showerror('Delete Error', "Selected entry could not be deleted due to error:-" + str(e))
                return

    """ Method - menuRefreshScriptData
        Looks up the market data and refreshes the same along with updating existing holding records"""
    def menuRefreshScriptData(self):
        script_name = self.output_tree.get_parent_item()
        if(len(script_name) <=0):
            msgbx.showwarning("Warning", "Please select valid row")
            return        
        self.output_tree.get_stock_quote("", script_name, DataFrame(),"Purchase Price="+'', "Purchase Date="+'',
                        "Purchase Qty="+'', "Commission Paid="+'', "Cost of Investment="+'')


    """menuModifySelectedScript"""
    def menuModifySelectedScript(self):
        rowid = self.output_tree.is_market_holding_col_row()
        
        if(len(rowid) <= 0):
            msgbx.showwarning("Modify Script", "Please select valid script row to modify")
            return
        else:
            script_name = self.output_tree.get_parent_item(rowid)
            if(len(script_name) <=0):
                msgbx.showwarning("Warning", "Please select valid row")
                return

        row_val = self.output_tree.item(rowid, 'values')
        
        dmodifyscript = dict()
        dmodifyscript = classAddNewModifyScript(master=self.root, argisadd=False, argscript=script_name, 
            argPurchasePrice=row_val[0], argPurchaseDate=row_val[1], argPurchaseQty=row_val[2], argCommissionPaid=row_val[3], argCostofInvestment=row_val[4]).show()
        # returns dictionary - {'Symbol': 'LT.BSE', 'Price': '1000', 'Date': '2020-02-22', 'Quantity': '10', 'Commission': '1', 'Cost': '10001.0'}
        if((dmodifyscript != None) and (len(dmodifyscript['Symbol']) >0)):
            stock_name = dmodifyscript['Symbol']
            listnewscript = list(dmodifyscript.items())
            self.output_tree.get_stock_quote(rowid, stock_name, DataFrame(), listnewscript[1][0] + '=' +listnewscript[1][1],
                                            listnewscript[2][0] + '=' + listnewscript[2][1],
                                            listnewscript[3][0] + '=' + listnewscript[3][1],
                                            listnewscript[4][0] + '=' + listnewscript[4][1],
                                            listnewscript[5][0] + '=' + listnewscript[5][1])

    """ Method - menuDeleteSelectedScript
        Deletes selection. If parent script is selected everything is deleted
        else individual row under parent is selected
        if selection is MARKETCOL or HOLDINGCOL then nothing will be deleted"""
    def menuDeleteSelectedScript(self):
        item = self.output_tree.is_market_holding_col_row()
        if(len(item) > 0):
            try:
                if(msgbx.askyesno('Delete script', 'Are you sure you want to delete: '+ item + '?')):
                    self.output_tree.delete(item)
                    self.output_tree.update()
                    msgbx.showinfo('Delete Script', "Selected entry deleted successfully. Please make sure to save updated portfolio!")
            except Exception as e:
                msgbx.showerror('Delete Error', "Selected entry could not be deleted due to error:-" + str(e))
                return

    def menuPortfolioPerformance(self):
        obj = PortfolioValuation(master=self.root, argkey=self.key, 
                    argscripttree=self.output_tree,argIsTest=self.bool_test, argDataFolder=self.datafolderpath)
        obj.ShowPortfolioPerformance()

    """ called on right click menu selection """
    def menuShowScriptPerformanceAllGraphs(self):
        script_name = self.output_tree.get_parent_item()
        if(len(script_name) <=0):
            msgbx.showwarning("Warning", "Please select valid row")
            return
        # Now get the purchase price if available
        holdinvalobj = BackTestSMA(master=self.root, argkey=self.key, argscript=script_name, 
                argscripttree=self.output_tree, argavgsmall=10, argavglarge=20, arglookbackyears=1,
                argIsTest=self.bool_test, argDataFolder=self.datafolderpath)
        holdinvalobj.findScriptPerformance(argShowPerformance=True, argShowCandlestick=True, 
                            argShowMarketData=True, argShowReturns=True)
        holdinvalobj.show()
        return

    """ called on right click menu selection """
    def menuShowScriptPerformance(self):
        script_name = self.output_tree.get_parent_item()
        if(len(script_name) <=0):
            msgbx.showwarning("Warning", "Please select valid row")
            return
        # Now get the purchase price if available
        holdinvalobj = BackTestSMA(master=self.root, argkey=self.key, argscript=script_name, 
                argscripttree=self.output_tree, argavgsmall=10, argavglarge=20, arglookbackyears=1,
                argIsTest=self.bool_test, argDataFolder=self.datafolderpath)
        holdinvalobj.findScriptPerformance(argShowPerformance=True, argShowCandlestick=False, 
                            argShowMarketData=False, argShowReturns=False)
        holdinvalobj.show()
        return

    """ called on right click menu selection """
    def menuShowScriptCandlestick(self):
        script_name = self.output_tree.get_parent_item()
        if(len(script_name) <=0):
            msgbx.showwarning("Warning", "Please select valid row")
            return
        # Now get the purchase price if available
        holdinvalobj = BackTestSMA(master=self.root, argkey=self.key, argscript=script_name, 
                argscripttree=self.output_tree, argavgsmall=10, argavglarge=20, arglookbackyears=1,
                argIsTest=self.bool_test, argDataFolder=self.datafolderpath)
        holdinvalobj.findScriptPerformance(argShowPerformance=False, argShowCandlestick=True, 
                            argShowMarketData=False, argShowReturns=False)
        holdinvalobj.show()
        return

    """ called on right click menu selection """
    def menuShowScriptBuySell(self):
        script_name = self.output_tree.get_parent_item()
        if(len(script_name) <=0):
            msgbx.showwarning("Warning", "Please select valid row")
            return
        # Now get the purchase price if available
        holdinvalobj = BackTestSMA(master=self.root, argkey=self.key, argscript=script_name, 
                argscripttree=self.output_tree, argavgsmall=10, argavglarge=20, arglookbackyears=1,
                argIsTest=self.bool_test, argDataFolder=self.datafolderpath)
        holdinvalobj.findScriptPerformance(argShowPerformance=False, argShowCandlestick=False, 
                            argShowMarketData=True, argShowReturns=False)
        holdinvalobj.show()
        return

    """ called on right click menu selection """
    def menuShowScriptReturns(self):
        script_name = self.output_tree.get_parent_item()
        if(len(script_name) <=0):
            msgbx.showwarning("Warning", "Please select valid row")
            return
        # Now get the purchase price if available
        holdinvalobj = BackTestSMA(master=self.root, argkey=self.key, argscript=script_name, 
                argscripttree=self.output_tree, argavgsmall=10, argavglarge=20, arglookbackyears=1,
                argIsTest=self.bool_test, argDataFolder=self.datafolderpath)
        holdinvalobj.findScriptPerformance(argShowPerformance=False, argShowCandlestick=False, 
                            argShowMarketData=False, argShowReturns=True)
        holdinvalobj.show()
        return

    def menuResearchGraphs(self):
        obj = classAllGraphs(master=self.root, argistestmode=self.bool_test, 
                argkey=self.key, argscript='', argmenucalled=True, arggraphid=-1, 
                argoutputtree=self.output_tree, argdatafolder=self.datafolderpath)
        obj.InitializeWindow()

    # command handler for stock quote button
    def menuGetStockQuote(self):
        obj = classGetQuote(master=self.root, argoutputtree=self.output_tree, argIsTest=self.bool_test, argDataFolder=self.datafolderpath)
        obj.show()
        return;

    # command handler for intra day
    def menuGetIntraDay(self):
        return True

    # command handler for daily stock
    def menuDailyStock(self):
        return True

    # This method is called from right click event
    def TreeViewSelectionChanged(self):
        
        tree_depth = self.output_tree.get_children() 
        if(len(tree_depth) > 0): #make sure call was not from resetExisting
            tempParent = self.output_tree.get_parent_item()
            if(len(self.currentScript) == 0):
                self.currentScript = tempParent
            elif (tempParent != self.currentScript): #check if any graphs are shown in figure & clear them if there are
                for i in range(len(self.ax)):
                    self.ax[i].clear()
                    self.ax[i].set_visible(False)
                self.dfDaily = DataFrame()
                self.dfSMA = DataFrame()
                self.dfIntra = DataFrame()
                self.dfVWMP = DataFrame()
                self.dfRSI = DataFrame()
                self.dfStoch = DataFrame()
                self.dfMACD = DataFrame()
                self.dfAROON = DataFrame()
                self.dfBBANDS = DataFrame()
                self.dfADX = DataFrame()

                self.mouseClickMoveEnableDisable(False)
                self.graphctr = 1
                self.currentScript = tempParent
                self.f.clear()
                self.setFigureCommonConfig(self.currentScript)
                #self.f.suptitle("", size='small')
                return True
        else:
            self.currentScript = ''
            for i in range(len(self.ax)):
                self.ax[i].clear()
                self.ax[i].set_visible(False)
            self.dfDaily = DataFrame()
            self.dfSMA = DataFrame()
            self.dfIntra = DataFrame()
            self.dfVWMP = DataFrame()
            self.dfRSI = DataFrame()
            self.dfStoch = DataFrame()
            self.dfMACD = DataFrame()
            self.dfAROON = DataFrame()
            self.dfBBANDS = DataFrame()
            self.dfADX = DataFrame()

            self.mouseClickMoveEnableDisable(False)
            self.graphctr = 1
            self.f.clear()
            self.setFigureCommonConfig(self.currentScript)
            return True
        return False

    # This method is called by each right click plot menu handlers when menu variable is False
    # indicating user does not want to see the specific menu graph.
    # Here we reduce index of all graphs, whoose index is greater than the current, by one
    def clearandresetGraphs(self, argDictIndex, argDictKey):
        #ax = plt.subplot(111)
        #ax.change_geometry(3,1,1)
        #first clear the current deselected graph
        try:
            self.ax[argDictIndex].clear()
            self.ax[argDictIndex].set_visible(False)
            self.f.delaxes(self.ax[argDictIndex])

            if(self.graphctr > 1):
                currdict = self.dictgraphmenu[argDictIndex]
                for each in range(0, len(self.dictgraphmenu)): 
                    tempdict = self.dictgraphmenu[each]
                    for key in tempdict:
                        if( currdict[argDictKey][2] < tempdict[key][2]):
                            tempdict[key] = (tempdict[key][0], tempdict[key][1], tempdict[key][2]-1)
                            self.ax[each].change_geometry(tempdict[key][0], tempdict[key][1], tempdict[key][2])

                self.graphctr -= 1
            elif(self.graphctr == 1):
                self.mouseClickMoveEnableDisable(False)
        except Exception as e:
            msgbx.showerror("Clear & Reset Graph", "Exception: " + str(e))
            return False
        return True

    # argLookbackYears - is the no of years we want to go back from today
    # if today is 2020-03-23 & argLookbackYears = 1, return will be 2019-03-23
    # the expetion takes care of leap year
    def getPastDateFromToday(self, argLookbackYears):
        try:
            dt = date.today()
            dt = dt.replace(year=dt.year-argLookbackYears)
        except ValueError:
            dt = dt.replace(year=dt.year-argLookbackYears, day=dt.day-1)
        return str(dt)
    
    #common method to set figure variables called by each right click plot menu handlers
    def setFigureCommonConfig(self, script_name):
        self.f.suptitle(script_name, size='xx-small', y=.996, weight='semibold')
        #self.f.tight_layout()
        #self.f.legend(loc='upper right')

        #self.output_canvas.set_window_title(script_name)
        # toolbar=NavigationToolbar2Tk(output_canvas, output_canvas.get_tk_widget())
        self.output_canvas.draw()
        self.toolbar.update()

    #common method called by each right click menu handlers to set respective Axes properties
    # when a new graph is to be shown
    def setAxesCommonConfig(self, argAxesIndex, argAxesKey, argScriptName, argTitle):
        if(self.graphctr == 1):
            self.mouseClickMoveEnableDisable(True)
        self.dictgraphmenu[argAxesIndex][argAxesKey] = (self.dictgraphmenu[argAxesIndex][argAxesKey][0], self.dictgraphmenu[argAxesIndex][argAxesKey][1], self.graphctr)
        self.graphctr += 1

        self.ax[argAxesIndex].tick_params(direction='out', length=6, width=2, colors='black',
            grid_color='black', grid_alpha=0.5, labelsize='xx-small')
        self.ax[argAxesIndex].tick_params(axis='x', labelrotation=30)

        self.ax[argAxesIndex].grid(True)
        self.ax[argAxesIndex].set_title(argTitle, size='xx-small')
        self.ax[argAxesIndex].legend(fontsize='xx-small')

    #Helper method to reverse the state of right click menu
    def reverseMenutick(self, argCurrentMenuState):
        if(argCurrentMenuState.get() == True):
            return False
        return True
    
    #When user selects a new script from Tree View, this method will set the right click 
    # menu handlers to False, called from right click even handlers
    def resetMenuticktoFalse(self):
        self.POSrightclickmenuDailyVsSMA.set(False)
        self.POSrightclickmenuAROON.set(False)
        self.POSrightclickmenuBBands.set(False)
        self.POSrightclickmenuIntraDay.set(False)
        self.POSrightclickmenuMACD.set(False)
        self.POSrightclickmenuRSI.set(False)
        self.POSrightclickmenuADX.set(False)
        self.POSrightclickmenuStochasticOscillator.set(False)
        self.POSrightclickmenuVWAP.set(False)

    # Method to bind & unbind the mouse move & mouse click events in plot area
    # Called from setAxesCommonConfig when graphctr = 1 to bind the events
    # Called from clearandresetGraphs when graphctr = 1 to unbind
    def mouseClickMoveEnableDisable(self, argbFlag):
        if(argbFlag == True):
            #self.cid_leftclick = self.output_canvas.callbacks.connect('button_press_event', self.on_click_graphs)
            #self.cid_leftrelease = self.output_canvas.callbacks.connect('button_release_event', self.on_release_graphs)
            self.cid_mouse_move = self.output_canvas.callbacks.connect('motion_notify_event', self.on_mouse_move)
        else:
            #if(self.cid_leftclick != None):
            #    self.output_canvas.callbacks.disconnect(self.cid_leftclick)
            #self.cid_leftclick = None
            #if(self.cid_leftrelease != None):
            #    self.output_canvas.callbacks.disconnect(self.cid_leftrelease)
            #self.cid_leftrelease = None
            self.xmouseposition = None
            self.ymouseposition = None
            if(self.cid_mouse_move != None):
                self.output_canvas.callbacks.disconnect(self.cid_mouse_move)
            self.cid_mouse_move = None

    """ Method - rightclickmenuDailyVsSMA
        Mouse right click- Method shows daily timeseries for selected stock within the app window"""
    def rightclickmenuDailyVsSMA(self):
        script_name = self.output_tree.get_parent_item()

        if(len(script_name) <=0):
            msgbx.showwarning("Warning", "Please select valid row")
            self.POSrightclickmenuDailyVsSMA.set(self.reverseMenutick(self.POSrightclickmenuDailyVsSMA))
            return

        #menutext = self.popup_menu_righclick.entrycget(self.POSrightclickmenuDailyVsSMA, 'label')
        #menu.entryconfigure(self.POSrightclickmenuDailyVsSMA, label=menutext[1:])
        
        if(self.POSrightclickmenuDailyVsSMA.get() == False):
            self.clearandresetGraphs(0, 'm1')
            self.setFigureCommonConfig(script_name)
            return
        
        # Get the data, returns a tuple
        # aapl_data is a pandas dataframe, aapl_meta_data is a dict
        try:
            if self.bool_test:
                testobj = PrepareTestData(argFolder=self.datafolderpath, argOutputSize='compact')
                self.dfDaily = testobj.loadDaily(script_name)
                if(self.dfSMA.empty):
                    self.dfSMA = testobj.loadSMA(script_name)
            else:
                self.dfDaily, aapl_meta_data = self.ts.get_daily(symbol=script_name)
                if(self.dfSMA.empty):
                    self.dfSMA, aapl_meta_sma = self.ti.get_sma(symbol=script_name)

            #the daily compact data has data for 100 days
            self.dfDaily=self.dfDaily.sort_index(axis=0, ascending=False)
            self.dfSMA=self.dfSMA.sort_index(axis=0, ascending=False)
            #Commenting as we are not taking FULL data from get_daily
            #pastdate = self.getPastDateFromToday(self.LookbackYears)
            #aapl_data=aapl_data.loc[aapl_data.index[:] >= pastdate]
            #aapl_sma=aapl_sma.loc[aapl_sma.index[:] >= pastdate]

            #commenting the annotation code below as the graph is too small to annotate
            """listpurchasprice = list()
            childrows = self.output_tree.get_children(script_name)
            for child in childrows:
                # now get  rows values only for self holding, we will not store market data
                if(str(child).upper().find(self.output_tree.HOLDINGVAL) >= 0):
                    child_val = self.output_tree.item(child, 'values')
                    listpurchasprice.append([child_val[0], child_val[1]])"""

            sizeofdaily = self.dfDaily.index.size
            #self.dfSMA = self.dfSMA.head(sizeofdaily)
            # Visualization
            self.ax[0].clear()
            #self.ax[0].set_visible(True)
            self.ax[0] = self.f.add_subplot(self.dictgraphmenu[0]['m1'][0], self.dictgraphmenu[0]['m1'][1], self.graphctr, gid=0, visible=True, label='Daily Close Vs SMA')#, title=script_name, label='Daily close price', xlabel='Date', ylabel='Closing price')
            self.ax[0].plot(self.dfDaily['4. close'], label='Close', gid=0, marker='*', markersize = 5, markevery=5, picker=5)
            #self.ax[0].plot(self.dfDaily['4. close'], 'x', markersize=3)
            self.ax[0].plot(self.dfSMA.head(sizeofdaily)['SMA'], gid=0, label='20 SMA', picker=5)
            self.ax[0].lines[0].set_pickradius(1)
            self.ax[0].lines[1].set_pickradius(2)

            #commenting annotations
            """for eachrow in listpurchasprice:
                if ((eachrow[0] != '') and (eachrow[1] != '')):
                    self.ax[0].annotate(eachrow[0], (mdates.datestr2num(eachrow[1]), float(eachrow[0])),
                        xytext=(15,15), textcoords='offset points', arrowprops=dict(arrowstyle='-|>'))"""

            self.setAxesCommonConfig(0, 'm1', script_name, 'Daily Vs 20 SMA')
            #self.f.autofmt_xdate()
            self.setFigureCommonConfig(script_name)
        except Exception as e:
            msgbx.showerror('Error', 'Exception in Daily Vs SMA: ' + str(e))
            self.POSrightclickmenuDailyVsSMA.set(self.reverseMenutick(self.POSrightclickmenuDailyVsSMA))

    """ Method - rightclickmenuIntraDay
        Mouse right click- Method shows  intraday close timeseries for selected stock within the app window"""
    def rightclickmenuIntraDay(self):
        script_name = self.output_tree.get_parent_item()
        if(len(script_name) <=0):
            msgbx.showwarning("Warning", "Please select valid row")
            self.POSrightclickmenuIntraDay.set(self.reverseMenutick(self.POSrightclickmenuIntraDay))
            return

        if(self.POSrightclickmenuIntraDay.get() == False):
            self.clearandresetGraphs(1, 'm2')
            self.setFigureCommonConfig(script_name)
            return

        # Get the data, returns a tuple
        # aapl_data is a pandas dataframe, aapl_meta_data is a dict
        try:
            if(self.bool_test == True):
                testobj = PrepareTestData(argFolder=self.datafolderpath)
                if(self.dfIntra.empty):
                    self.dfIntra = testobj.loadIntra(script_name)
            else:
                if(self.dfIntra.empty):
                    self.dfIntra, aapl_meta_data = self.ts.get_intraday(symbol=script_name, interval='5min')

            self.dfIntra=self.dfIntra.sort_index(axis=0, ascending=False)
            latestdt = self.dfIntra.index[0]
            latestdt = str(str(latestdt.year) + '-' + str(latestdt.month) + '-' + str(latestdt.day) + ' 00:00:00')
            self.dfIntra = self.dfIntra.loc[self.dfIntra.index[:] >= latestdt]            
            # Visualization
            self.ax[1].clear()
            self.ax[1] = self.f.add_subplot(self.dictgraphmenu[1]['m2'][0], self.dictgraphmenu[1]['m2'][1], self.graphctr, gid=1, visible=True, label='Intra-Day')#, title=script_name, label='Intra-day', xlabel='Date', ylabel='Intra-day close', visible=True)
            self.ax[1].plot(self.dfIntra['4. close'], gid=1, label='Intra-day', marker='x', markersize=5, markevery=5, picker = 5)

            self.setAxesCommonConfig(1, 'm2', script_name, 'Intra-day')
            self.setFigureCommonConfig(script_name)
        except Exception as e:
            msgbx.showerror("Error", "Error in IntraDay: " + str(e))
            self.POSrightclickmenuIntraDay.set(self.reverseMenutick(self.POSrightclickmenuIntraDay))

    def rightclickmenuVWAP(self):
        script_name = self.output_tree.get_parent_item()
        if(len(script_name) <=0):
            msgbx.showwarning("Warning", "Please select valid row")
            self.POSrightclickmenuVWAP.set(self.reverseMenutick(self.POSrightclickmenuVWAP))
            return

        if(self.POSrightclickmenuVWAP.get() == False):
            self.clearandresetGraphs(2, 'm3')
            self.setFigureCommonConfig(script_name)
            return
        try:
            if self.bool_test:
                testobj = PrepareTestData(argFolder=self.datafolderpath)
                self.dfVWMP = testobj.loadVWMP(script_name)
            else:
                self.dfVWMP, meta_vwap = self.ti.get_vwap(symbol=script_name)
            
            self.dfVWMP=self.dfVWMP.sort_index(axis=0, ascending=False)
            latestdt = self.dfVWMP.index[0]
            latestdt = str(str(latestdt.year) + '-' + str(latestdt.month) + '-' + str(latestdt.day) + ' 00:00:00')
            self.dfVWMP = self.dfVWMP.loc[self.dfVWMP.index[:] >= latestdt]            

            # Visualization
            self.ax[2].clear()
            #self.ax[2].set_visible(True)
            self.ax[2] = self.f.add_subplot(self.dictgraphmenu[2]['m3'][0], self.dictgraphmenu[2]['m3'][1], self.graphctr, gid=2, visible=True, label='VWAP')
                #, title=script_name, label='Intra-day', xlabel='Date', ylabel='Intra-day close', visible=True)
            sdateyearback = self.getPastDateFromToday(self.LookbackYears)
            self.ax[2].plot(self.dfVWMP.loc[self.dfVWMP.index[:] >= sdateyearback, 'VWAP'], gid=2,label='VWAP', picker=5)
            self.setAxesCommonConfig(2, 'm3', script_name, 'Vol Wt Avg Price')
            self.setFigureCommonConfig(script_name)
        except Exception as e:
            msgbx.showerror("Error", "Error in VWAP: " + str(e))
            self.POSrightclickmenuVWAP.set(self.reverseMenutick(self.POSrightclickmenuVWAP))

    def rightclickmenuRSI(self):
        script_name = self.output_tree.get_parent_item()
        if(len(script_name) <=0):
            msgbx.showwarning("Warning", "Please select valid row")
            self.POSrightclickmenuRSI.set(self.reverseMenutick(self.POSrightclickmenuRSI))
            return

        if(self.POSrightclickmenuRSI.get() == False):
            self.clearandresetGraphs(3, 'm4')
            self.setFigureCommonConfig(script_name)
            return
        try:
            if self.bool_test:
                testobj = PrepareTestData(argFolder=self.datafolderpath)
                self.dfRSI = testobj.loadRSI(script_name)
            else:
                self.dfRSI, meta_rsi = self.ti.get_rsi(symbol=script_name)
            
            self.dfRSI = self.dfRSI.sort_index(axis=0, ascending=False)
            #self.dfRSI = self.dfRSI.head(sizeofintra)
            # Visualization
            self.ax[3].clear()
            #self.ax[3].set_visible(True)
            #color = 'tab:red'
            self.ax[3] = self.f.add_subplot(self.dictgraphmenu[3]['m4'][0], self.dictgraphmenu[3]['m4'][1], self.graphctr, gid=3, visible=True, label='RSI')#, title=script_name, label='Intra-day', xlabel='Date', ylabel='Intra-day close', visible=True)
            #self.ax[3].tick_params(axis='y', labelcolor=color)
            #twinax = self.ax[3].twinx()
            #color = 'tab:blue'
            sdateyearback = self.getPastDateFromToday(self.LookbackYears)
            self.ax[3].plot(self.dfRSI.loc[self.dfRSI.index[:] >= sdateyearback, 'RSI'], gid=3,label='RSI', picker=5)
            #twinax.tick_params(axis='y', labelcolor=color)

            self.setAxesCommonConfig(3, 'm4', script_name, 'RSI')
            self.setFigureCommonConfig(script_name)
        except Exception as e:
            msgbx.showerror("Error", "Error in RSI: " + str(e))
            self.POSrightclickmenuVWAP.set(self.reverseMenutick(self.POSrightclickmenuVWAP))

    def rightclickmenuADX(self):
        script_name = self.output_tree.get_parent_item()
        if(len(script_name) <=0):
            msgbx.showwarning("Warning", "Please select valid row")
            self.POSrightclickmenuADX.set(self.reverseMenutick(self.POSrightclickmenuADX))
            return

        if(self.POSrightclickmenuADX.get() == False):
            self.clearandresetGraphs(4, 'm5')
            self.setFigureCommonConfig(script_name)
            return
        
        try:
            if self.bool_test:
                testobj = PrepareTestData(argFolder=self.datafolderpath)
                self.dfADX = testobj.loadADX(script_name)
            else:
                self.dfADX, meta_adx = self.ti.get_adx(symbol=script_name)
                
            self.dfADX = self.dfADX.sort_index(axis=0, ascending=False)

            # Visualization
            self.ax[4].clear()
            #self.ax[4].set_visible(True)
            self.ax[4] = self.f.add_subplot(self.dictgraphmenu[4]['m5'][0], self.dictgraphmenu[4]['m5'][1], self.graphctr, gid=4, visible=True, label='ADX')
            #, title=script_name, label='Intra-day', xlabel='Date', ylabel='Intra-day close', visible=True)
            sdateyearback = self.getPastDateFromToday(self.LookbackYears)
            self.ax[4].plot(self.dfADX.loc[self.dfADX.index[:] >= sdateyearback, 'ADX'], gid=4,label='ADX', picker=5)
            self.setAxesCommonConfig(4, 'm5', script_name, 'ADX')
            self.setFigureCommonConfig(script_name)
        except Exception as e:
            msgbx.showerror("Error", "Error in ADX: " + str(e))
            self.POSrightclickmenuADX.set(self.reverseMenutick(self.POSrightclickmenuADX))

    def rightclickmenuStochasticOscillator(self):
        script_name = self.output_tree.get_parent_item()
        if(len(script_name) <=0):
            msgbx.showwarning("Warning", "Please select valid row")
            self.POSrightclickmenuStochasticOscillator.set(self.reverseMenutick(self.POSrightclickmenuStochasticOscillator))
            return

        if(self.POSrightclickmenuStochasticOscillator.get() == False):
            self.clearandresetGraphs(5, 'm6')
            self.setFigureCommonConfig(script_name)
            return
        try:
            if self.bool_test:
                testobj = PrepareTestData(argFolder=self.datafolderpath)
                self.dfStoch = testobj.loadStochasticOscillator(script_name)
            else:
                self.dfStoch, meta_stoch = self.ti.get_stoch(symbol=script_name, interval='daily',
                    fastkperiod=5, slowkperiod=3, slowdperiod=3, slowkmatype=0, slowdmatype=0)
                #data_stoch = data_stoch.sort_index(axis=0, ascending=False)

            self.dfStoch = self.dfStoch.sort_index(axis=0, ascending=False)
            # Visualization
            self.ax[5].clear()
            #self.ax[5].set_visible(True)
            self.ax[5] = self.f.add_subplot(self.dictgraphmenu[5]['m6'][0], self.dictgraphmenu[5]['m6'][1], self.graphctr, gid=5, visible=True, label='STOCH')
            #, title=script_name, label='Intra-day', xlabel='Date', ylabel='Intra-day close', visible=True)
            sdateyearback = self.getPastDateFromToday(self.LookbackYears)
            self.ax[5].plot(self.dfStoch.loc[self.dfStoch.index[:] >= sdateyearback, 'SlowK'], 'b-', gid=5, label='SlowK MA', picker=5)
            self.ax[5].plot(self.dfStoch.loc[self.dfStoch.index[:] >= sdateyearback, 'SlowD'], 'r-', gid=5, label='SlowD MA', picker=5)
            self.setAxesCommonConfig(5, 'm6', script_name, 'Stoch Oscillator')
            self.setFigureCommonConfig(script_name)
        except Exception as e:
            msgbx.showerror("Error", "Error in STOCH: " + str(e))
            self.POSrightclickmenuStochasticOscillator.set(self.reverseMenutick(self.POSrightclickmenuStochasticOscillator))

    def rightclickmenuMACD(self):
        script_name = self.output_tree.get_parent_item()
        if(len(script_name) <=0):
            msgbx.showwarning("Warning", "Please select valid row")
            self.POSrightclickmenuMACD.set(self.reverseMenutick(self.POSrightclickmenuMACD))
            return

        if(self.POSrightclickmenuMACD.get() == False):
            self.clearandresetGraphs(6, 'm7')
            self.setFigureCommonConfig(script_name)
            return
        try:
            if self.bool_test:
                testobj = PrepareTestData(argFolder=self.datafolderpath)
                self.dfMACD = testobj.loadMACD(script_name)
            else:
                self.dfMACD, meta_macd = self.ti.get_macd(symbol=script_name, interval='daily',
                    series_type='close', fastperiod=12, slowperiod=26, signalperiod=9)

            self.dfMACD = self.dfMACD.sort_index(axis=0, ascending=False)
            # Visualization
            self.ax[6].clear()
            #self.ax[6].set_visible(True)
            self.ax[6] = self.f.add_subplot(self.dictgraphmenu[6]['m7'][0], self.dictgraphmenu[6]['m7'][1], self.graphctr, gid=6, visible=True, label='MACD')
            #, title=script_name, label='Intra-day', xlabel='Date', ylabel='Intra-day close', visible=True)
            sdateyearback = self.getPastDateFromToday(self.LookbackYears)
            self.ax[6].plot(self.dfMACD.loc[self.dfMACD.index[:] >= sdateyearback, 'MACD_Signal'], 'r-', gid=6, label='Signal', picker=5)
            self.ax[6].plot(self.dfMACD.loc[self.dfMACD.index[:] >= sdateyearback, 'MACD'], 'y-', gid=6, label='MACD', picker=5)
            self.ax[6].plot(self.dfMACD.loc[self.dfMACD.index[:] >= sdateyearback, 'MACD_Hist'], 'b-', gid=6, label='History', picker=5)
            self.setAxesCommonConfig(6, 'm7', script_name, 'MACD')
            self.setFigureCommonConfig(script_name)
        except Exception as e:
            msgbx.showerror("Error", "Error in MACD: "+ str(e))
            self.POSrightclickmenuMACD.set(self.reverseMenutick(self.POSrightclickmenuMACD))
    
    def rightclickmenuAROON(self):
        script_name = self.output_tree.get_parent_item()
        if(len(script_name) <=0):
            msgbx.showwarning("Warning", "Please select valid row")
            self.POSrightclickmenuAROON.set(self.reverseMenutick(self.POSrightclickmenuAROON))
            return

        if(self.POSrightclickmenuAROON.get() == False):
            self.clearandresetGraphs(7, 'm8')
            self.setFigureCommonConfig(script_name)
            return
        try:
            if self.bool_test:
                testobj = PrepareTestData(argFolder=self.datafolderpath)
                self.dfAROON = testobj.loadAROON(script_name)
            else:
                self.dfAROON, meta_aroon = self.ti.get_aroon(symbol=script_name)
            
            self.dfAROON = self.dfAROON.sort_index(axis=0, ascending=False)

            # Visualization
            self.ax[7].clear()
            #self.ax[7].set_visible(True)
            self.ax[7] = self.f.add_subplot(self.dictgraphmenu[7]['m8'][0], self.dictgraphmenu[7]['m8'][1], self.graphctr, gid=7, visible=True, label='AROON')
                #, title=script_name, label='Intra-day', xlabel='Date', ylabel='Intra-day close', visible=True)
            sdateyearback = self.getPastDateFromToday(self.LookbackYears)
            self.ax[7].plot(self.dfAROON.loc[self.dfAROON.index[:] >= sdateyearback, 'Aroon Up'], 'r-', gid=7, label='Up', picker=5)
            self.ax[7].plot(self.dfAROON.loc[self.dfAROON.index[:] >= sdateyearback, 'Aroon Down'], 'b-', gid=7, label='Down', picker=5)
            self.setAxesCommonConfig(7, 'm8', script_name, 'Aroon')
            self.setFigureCommonConfig(script_name)
        except Exception as e:
            msgbx.showerror("Error", "Error in AROON: " + str(e))
            self.POSrightclickmenuAROON.set(self.reverseMenutick(self.POSrightclickmenuAROON))
    
    def rightclickmenuBBands(self):
        script_name = self.output_tree.get_parent_item()
        if(len(script_name) <=0):
            msgbx.showwarning("Warning", "Please select valid row")
            self.POSrightclickmenuBBands.set(self.reverseMenutick(self.POSrightclickmenuBBands))
            return

        if(self.POSrightclickmenuBBands.get() == False):
            self.clearandresetGraphs(8, 'm9')
            self.setFigureCommonConfig(script_name)
            return
        
        try:
            if self.bool_test:
                testobj = PrepareTestData(argFolder=self.datafolderpath)
                self.dfBBANDS=testobj.loadBBands(script_name)
            else:
                self.dfBBANDS, meta_bbands = self.ti.get_bbands(symbol=script_name, interval='daily',
                    time_period=20, series_type='close',nbdevup=2, nbdevdn=2, matype=0)
            
            self.dfBBANDS = self.dfBBANDS.sort_index(axis=0, ascending=False)

            # Visualization
            self.ax[8].clear()
            #self.ax[8].set_visible(True)
            self.ax[8] = self.f.add_subplot(self.dictgraphmenu[8]['m9'][0], self.dictgraphmenu[8]['m9'][1], self.graphctr, gid=8, visible=True, label='BBANDS')
            #, title=script_name, label='Intra-day', xlabel='Date', ylabel='Intra-day close', visible=True)
            sdateyearback = self.getPastDateFromToday(self.LookbackYears)
            self.ax[8].plot(self.dfBBANDS.loc[self.dfBBANDS.index[:] >= sdateyearback, 'Real Middle Band'], 'r-', gid=8, label='Middle', picker=5)
            self.ax[8].plot(self.dfBBANDS.loc[self.dfBBANDS.index[:] >= sdateyearback, 'Real Upper Band'], 'b-', gid=8, label='Upper', picker=5)
            self.ax[8].plot(self.dfBBANDS.loc[self.dfBBANDS.index[:] >= sdateyearback, 'Real Lower Band'], 'y-', gid=8, label='Lower', picker=5)
            self.setAxesCommonConfig(8, 'm9', script_name, 'Bollinger Bands')
            self.setFigureCommonConfig(script_name)
        except Exception as e:
            msgbx.showerror("Error", "Error in BBANDS: " + str(e))
            self.POSrightclickmenuAROON.set(self.reverseMenutick(self.POSrightclickmenuAROON))


    def testCompareSMANOTUSED(self):
        script_name = self.output_tree.get_parent_item()
        if(len(script_name) <=0):
            msgbx.showwarning("Warning", "Please select valid row")
            return
        # Now get the purchase price if available
        listpurchasprice = list()
        childrows = self.output_tree.get_children(script_name)
        for child in childrows:
            # now get  rows values only for self holding, we will not store market data
            if(str(child).upper().find(self.output_tree.HOLDINGVAL) >= 0):
                child_val = self.output_tree.item(child, 'values')
                listpurchasprice.append([child_val[0], child_val[1]])
        try:
            #fig, ax = plt.subplot(3, 1, sharex=True, figsize=[16, 9])

            aapl_data = DataFrame()
            aapl_sma = DataFrame()
            aapl_data, aapl_meta_data = self.ts.get_daily(symbol=script_name)
            aapl_sma, aapl_meta_sma = self.ti.get_sma(symbol=script_name)
            
            sizeofdaily = aapl_data.index.size
            aapl_sma = aapl_sma.tail(sizeofdaily)

            f_temp=Figure(figsize=(15, 6), dpi=80, facecolor='w', edgecolor='k')

            #portfolio value
            #ax1 = plt.subplot(211)
            ax1 = plt.subplot(111)
            plt.plot(aapl_data['4. close'], label='Daily stock price')
            #plt.ylabel('Daily Stock Price')
            #plt.legend(loc='upper left')
            plt.legend()
            plt.grid()
            for eachrow in listpurchasprice:
                if ((eachrow[0] != '') and (eachrow[1] != '')):
                    plt.annotate(eachrow[0], (mdates.datestr2num(eachrow[1]), float(eachrow[0])),
                        xytext=(15,15), textcoords='offset points', arrowprops=dict(arrowstyle='-|>'))
                #plt.axhline(float(eachrow[0]), color='y') # will draw a horizontal line at purchase price
                #plt.axvline(mdates.datestr2num(eachrow[1]), color='y')

            #plt.subplot(212, sharex=ax1)
            plt.plot(aapl_sma['SMA'], label='SMA')

            #plt.xlabel('Date')
            #plt.ylabel('Simple Moving Average')
            #plt.legend(loc='upper left')
            plt.legend()
            plt.grid()

            plt.suptitle(script_name)
            plt.show()
        except ValueError as error:
            msgbx.showerror("Alpha Vantage error", error)
            return

    """ Method - menuComparePriceSMA
        Method to show comparison bewtween daily timeseriese and SMA"""
    def menuComparePriceSMANOTUSED(self):
        self.testCompareSMA()
        return
        script_name = self.output_tree.get_parent_item()
        if(len(script_name) <=0):
            msgbx.showwarning("Warning", "Please select valid row")
            return
        # Now get the purchase price if available
        listpurchasprice = list()
        childrows = self.output_tree.get_children(script_name)
        for child in childrows:
            # now get  rows values only for self holding, we will not store market data
            if(str(child).upper().find(self.output_tree.HOLDINGVAL) >= 0):
                child_val = self.output_tree.item(child, 'values')
                listpurchasprice.append([child_val[0], child_val[1]])

        if self.bool_test:
            aapl_data = pd.read_csv("E:\\python_projects\\TestData\\daily_MSFT.csv")
        else:
            try:
                aapl_data, aapl_meta_data = self.ts.get_daily(symbol=script_name)
                # Not sure if we need the following line -- commenting for time being
                # aapl_sma is a df, aapl_meta_sma also a dict
                aapl_sma, aapl_meta_sma = self.ti.get_sma(symbol=script_name)
                
            except ValueError as error:
                msgbx.showerror("Alpha Vantage error", error)
                return

        # get users price & date
        #dict_curr_row = self.output_tree.item(script_name)
        #purchase_price = dict_curr_row['values'][1]
        #purchase_date =  dict_curr_row['values'][2]
        
        # Visualization
        f_temp=Figure(figsize=(15, 6), dpi=80, facecolor='w', edgecolor='k')

        plt.clf()
        if(self.bool_test):
            # aapl_data['close'].plot(title=script_name)
            plt.plot(aapl_data['close'], label='Stock price')
        else:
            
            #aapl_data['4. close'].plot(title=script_name)
            plt.plot(aapl_data['4. close'], label='Stock price')
            plt.plot(aapl_sma['SMA'], label='SMA')

        plt.title(script_name)
        plt.xlabel('Date')
        plt.ylabel('Price')
        for eachrow in listpurchasprice:
            if ((eachrow[0] != '') and (eachrow[1] != '')):
                plt.annotate('Your price point', (mdates.datestr2num(eachrow[1]), float(eachrow[0])),
                    xytext=(15,15), textcoords='offset points', arrowprops=dict(arrowstyle='-|>'))
                plt.axhline(float(eachrow[0]), color='y') # will draw a horizontal line at purchase price
                plt.axvline(mdates.datestr2num(eachrow[1]), color='y')

        plt.tight_layout()
        plt.legend(loc='upper left')
        plt.grid()
        plt.show()

    #allows the user to enter valid key
    def menuKeyDataFolder(self):
        result_dict = classAddKey(master=self.root).show()
        if(len(result_dict) > 0):
            if(result_dict['key'] != ''):
                self.key = result_dict['key']
                self.ts = TimeSeries(self.key, output_format='pandas')
                self.ti = TechIndicators(self.key, output_format='pandas')
            if(result_dict['folder'] != ''):
                self.datafolder = result_dict['folder']

    def menuDownloadData(self):
        obj1 = classDownloadData(argKey=self.key)

    def menuSetTestMode(self):
        if (self.bool_test):
            self.bool_test = False
            self.output_tree.btestmode = False
            self.root.title('Stock Analytics - Online mode')
            self.root.update()
        else:
            self.bool_test = True
            self.output_tree.btestmode = True
            self.root.title('Stock Analytics - Offline mode')
            self.root.update()

        # File open menu handler
        # the file will be in following format
        #exchange:scriptname,Purchase Price=###.##,Purchase Date=YYYY-MM-DD,Purchase Qty=##.##,Commission Paid=##.##,Cost of Investment=####.##

    """Method - menuNewPortfolio"""
    def menuNewPortfolio(self):
        self.resetExisting()

    """Method - menuOpenPortfolio
        Opens a valid portfolio file"""
    def menuOpenPortfolio(self):
        try:
            openfilehandle=askopenfile('r', initialdir = "/", title = "Open portfolio file to load portfolio",filetypes = (("csv files","*.csv"),("all files","*.*")) )
            if openfilehandle is not None:
                self.root.configure(cursor='wait')
                self.root.update()
                list_scripts=openfilehandle.readlines()
                openfilehandle.close()
                self.resetExisting()
                symbolname = ''
                dfstockname = None
                for script in list_scripts:
                    # -1 to remove the last '\n' and then split the string by ','
                    arg_list=str(script[:-1]).split(',')
                    if(len(arg_list) == 6):
                        if(symbolname!=str(arg_list[0])):
                            try:
                                symbolname = str(arg_list[0])
                                if(self.bool_test):
                                    testobj = PrepareTestData(argFolder=self.datafolderpath)
                                    dfstockname = testobj.GetQuoteEndPoint(symbolname)
                                else:
                                    dfstockname, meta_data = self.ts.get_quote_endpoint(symbolname)
                            except ValueError as error:
                                msgbx.showerror("Open file-Alpha Vantage Error", error)
                                return
                        self.output_tree.get_stock_quote("", str(arg_list[0]), dfstockname, str(arg_list[1]), str(arg_list[2]),
                        str(arg_list[3]), str(arg_list[4]), str(arg_list[5]))    
                    else:
                        self.root.configure(cursor='')
                        msgbx.showerror("Open portfolio", "Error->Input file not in correct format." +"\n" + "Each line must be in the format of ScriptName,PurchasePrice,PurchaseDate")
                        return
                self.root.configure(cursor='')
        except Exception as e:
            self.root.configure(cursor='')
            msgbx.showerror('Error', 'Error while opening file: ' + str(e))

    """ method menuSavePortfolio       
        # the tree is in following format for each script
        # + Script.exchange
        #       Market Data     col1 col2....
        #       Market Val      val1 val2....
        #       Holding Data    col1 col2....
        #       Holding Value_1 val1 val2....
        #       Holding Value_2 val1 val2....
        # the file will be in following format
        #exchange:scriptname,Purchase Price=###.##,Purchase Date=YYYY-MM-DD,Purchase Qty=##.##,Commission Paid=##.##,Cost of Investment=####.## """
    def menuSavePortfolio(self):
        savefilehandle = asksaveasfile(initialdir = "/",title = "Select file to save portfolio scripts",filetypes = (("csv files","*.csv"),("all files","*.*")))
        # savefilehandle = open(savefilename, 'w')
        scripttowrite = self.output_tree.get_children()
        for script in scripttowrite: 
            #get children of the current iid
            childrows = self.output_tree.get_children(script)
            for child in childrows:
                # now get  rows values only for self holding, we will not store market data
                if(str(child).upper().find(self.output_tree.HOLDINGVAL) >= 0):
                    child_val = self.output_tree.item(child, 'values')
                    strtowrite = script+",Purchase Price="+child_val[0]+",Purchase Date="+child_val[1]+",Purchase Qty="+child_val[2]+",Commission Paid="+child_val[3]+",Cost of Investment="+child_val[4]+"\n"
                    savefilehandle.writelines(strtowrite)
        
        savefilehandle.close()

    def show_popup(self, event):
        #self.popup_ongraph.post(argarrpoint[0], argarrpoint[1])
       if event.inaxes is not None:
            self.event = event
            self.popup_ongraph.post(event.guiEvent.x_root, event.guiEvent.y_root)

    def menuClearAllGraphs(self):
        if(self.POSrightclickmenuDailyVsSMA.get() == True):
            self.POSrightclickmenuDailyVsSMA.set(False)
            self.rightclickmenuDailyVsSMA()
        if(self.POSrightclickmenuIntraDay.get() == True):
            self.POSrightclickmenuIntraDay.set(False)
            self.rightclickmenuIntraDay()
        if(self.POSrightclickmenuVWAP.get() == True):
            self.POSrightclickmenuVWAP.set(False)
            self.rightclickmenuVWAP()
        if(self.POSrightclickmenuRSI.get() == True):
            self.POSrightclickmenuRSI.set(False)
            self.rightclickmenuRSI()
        if(self.POSrightclickmenuADX.get() == True):
            self.POSrightclickmenuADX.set(False)
            self.rightclickmenuADX()
        if(self.POSrightclickmenuStochasticOscillator.get() == True):
            self.POSrightclickmenuStochasticOscillator.set(False)
            self.rightclickmenuStochasticOscillator()
        if(self.POSrightclickmenuMACD.get() == True):
            self.POSrightclickmenuMACD.set(False)
            self.rightclickmenuMACD()
        if(self.POSrightclickmenuAROON.get() == True):
            self.POSrightclickmenuAROON.set(False)
            self.rightclickmenuAROON()
        if(self.POSrightclickmenuBBands.get() == True):
            self.POSrightclickmenuBBands.set(False)
            self.rightclickmenuBBands()

    def OnAnalysis(self):
        msgbx.showinfo("Analysis", "Analysis")

    def OnDetailGraph(self):
        obj = classAllGraphs(master=self.root, argistestmode=self.bool_test, argkey=self.key,
                    #argscript=self.currentScript, argmenucalled=False, arggraphid=self.event.inaxes.get_gid(),
                    argscript=self.currentScript, argmenucalled=False, arggraphid=self.event.artist.get_gid(),
                    argoutputtree=self.output_tree, argdatafolder=self.datafolderpath)
        obj.InitializeWindow()

    def OnClearGraph(self):
        currentartist = self.event.artist
        #self.popup_menu_righclick.event_generate('<<Menu Selected>>', 13)
        if( currentartist.get_gid() == 0):
            self.POSrightclickmenuDailyVsSMA.set(self.reverseMenutick(self.POSrightclickmenuDailyVsSMA))
            self.rightclickmenuDailyVsSMA()
        elif( currentartist.get_gid() == 1):
            self.POSrightclickmenuIntraDay.set(self.reverseMenutick(self.POSrightclickmenuIntraDay))
            self.rightclickmenuIntraDay()
        elif( currentartist.get_gid() == 2):
            self.POSrightclickmenuVWAP.set(self.reverseMenutick(self.POSrightclickmenuVWAP))
            self.rightclickmenuVWAP()
        elif( currentartist.get_gid() == 3):
            self.POSrightclickmenuRSI.set(self.reverseMenutick(self.POSrightclickmenuRSI))
            self.rightclickmenuRSI()
        elif( currentartist.get_gid() == 4):
            self.POSrightclickmenuADX.set(self.reverseMenutick(self.POSrightclickmenuADX))
            self.rightclickmenuADX()
        elif( currentartist.get_gid() == 5):
            self.POSrightclickmenuStochasticOscillator.set(self.reverseMenutick(self.POSrightclickmenuStochasticOscillator))
            self.rightclickmenuStochasticOscillator()
        elif( currentartist.get_gid() == 6):
            self.POSrightclickmenuMACD.set(self.reverseMenutick(self.POSrightclickmenuMACD))
            self.rightclickmenuMACD()
        elif( currentartist.get_gid() == 7):
            self.POSrightclickmenuAROON.set(self.reverseMenutick(self.POSrightclickmenuAROON))
            self.rightclickmenuAROON()
        elif( currentartist.get_gid() == 8):
            self.POSrightclickmenuBBands.set(self.reverseMenutick(self.POSrightclickmenuBBands))
            self.rightclickmenuBBands()

    def OnPicker(self, event):
        """currentartist = event.artist
        if( currentartist.get_gid() == 0):
            msgbx.showinfo('DailyVsSMA', 'DailyVsSMA')"""
        self.event = event
        self.popup_ongraph.post(event.guiEvent.x_root, event.guiEvent.y_root)

    def on_click_graphs(self, event):
        return
        self.event = event
        return
        #if event.inaxes is not None:
            #self.xmouseposition = event.x
            #self.ymouseposition = event.y

    def on_release_graphs(self, event):
        #if event.inaxes is not None:
        #if( (self.xmouseposition == event.x) and (self.ymouseposition == event.y)):
        return
        if( (self.event.x == event.x) and (self.event.y == event.y)):
            if event.inaxes is not None:
                self.event = event
                self.popup_ongraph.post(event.guiEvent.x_root, event.guiEvent.y_root)

            """#msgbx.showinfo('If', 'xdata='+str(event.xdata) + '   ydata= ' + str(event.ydata))
                graphid = -1
                if(event.inaxes == self.ax[0]):
                    graphid = 0 #Daily
                elif (event.inaxes == self.ax[1]):
                    graphid = 1 #intra
                elif (event.inaxes == self.ax[2]):
                    graphid = 3 #VWAP
                elif (event.inaxes == self.ax[3]):
                    graphid = 4 #RSI
                elif (event.inaxes == self.ax[4]):
                    graphid = 5 #ADX
                elif (event.inaxes == self.ax[5]):
                    graphid = 6 #STOCH
                elif (event.inaxes == self.ax[6]):
                    graphid = 7 #MACD
                elif (event.inaxes == self.ax[7]):
                    graphid = 8 #AROON
                elif (event.inaxes == self.ax[8]):
                    graphid = 9 #BBANDS
                obj = classAllGraphs(master=self.root, argistestmode=self.bool_test, argkey=self.key,
                            argscript=self.currentScript, argmenucalled=False, arggraphid=graphid,
                            argoutputtree=self.output_tree)
                obj.InitializeWindow()"""

    def isPointOnGraphLine(self, argXData, argYData, argDF, argColName):
        #found = csvdf[csvdf['4. close'].astype('str').str.contains('1623.6')]
        #amount = ('{:.2f}'.format(argYData))
        currdate = mdates.num2date(argXData)
        currdate = date(currdate.year, currdate.month, currdate.day)
        #found = argDF[self.dfDaily[argColName].astype('str').str.contains(str(amount))]
        seriesmatch = argDF.loc[self.dfDaily.index[:] == str(currdate), argColName]
        if(seriesmatch.empty == False):
            samount = str(argYData)
            found = samount.find(str(seriesmatch[0]))
            if(found >=0):
                return True
        return False

    def on_mouse_move(self, event):
        #first remove existing visible annotation
        if(self.annotate_visible):
            for i in range(len(self.ax)):
                if(self.annotate_state[i] != None):
                    self.annotate_state[i].set_visible(False)
                    annotations = [child for child in self.ax[i].get_children() if isinstance(child, matplotlib.text.Annotation)]
                    for t in annotations:
                        t.remove()
                    self.annotate_state[i] = None
            
            #self.ax[event.inaxes.get_gid()].draw_artist(lines[0])
            #fig.canvas.blit(axes[0].bbox)
            
            self.output_canvas.draw()

        self.annotate_visible = False

        #now check if point is on any of the lines
        if event.inaxes is not None:
            for line in self.ax[event.inaxes.get_gid()].lines:
                self.annotate_visible, indarray = line.contains(event)
                if(self.annotate_visible):
                    break
            if(self.annotate_visible):
                self.annotate_state[event.inaxes.get_gid()] = self.ax[event.inaxes.get_gid()].annotate('{:.2f}'.format(event.ydata), 
                        xy=(event.xdata, event.ydata), xycoords='data', 
                        xytext=(1, 1), textcoords='offset points',
                        #xytext=(event.xdata, event.ydata), textcoords='data',
                        horizontalalignment="left", 
                        #arrowprops=dict(arrowstyle="simple", connectionstyle="arc3,rad=-0.2"),
                        bbox=dict(boxstyle="round", facecolor="w", edgecolor="0.5", alpha=0.9), 
                        fontsize='xx-small')
                self.output_canvas.draw()
            

    def old_on_mouse_move(self, event):
        if event.inaxes is not None:
            #msgbx.showinfo('If', 'xdata='+str(event.xdata) + '   ydata= ' + str(event.ydata))
            print('in mouse move--' + str(time.thread_time()))
            for i in range(len(self.ax)):
                if(self.annotate_state[i] != None):
                    self.annotate_state[i].set_visible(False)
                print('after if annotate--' + str(time.thread_time()))
            
                if(event.inaxes == self.ax[i]):
                    #line1_data = self.ax[0].lines[0].get_xydata()
                    #line2_data = self.ax[0].lines[1].get_xydata()
                    #a = [float('{:.0f}'.format(event.xdata)), float('{:.4f}'.format(event.ydata))]
                    #if( (a in line1_data) or (a in line2_data)):
                    bline1 = False
                    for j in range(len(self.ax[i].lines)):
                        print('before contain--' + str(time.thread_time()))
                        bline1, indarray = self.ax[i].lines[j].contains(event)
                        print('after contain--' + str(time.thread_time()))
                        if(bline1):
                            break
                    if(bline1):
                        print('before annotate--' + str(time.thread_time()))
                        self.annotate_state[i] = self.ax[i].annotate('{:.2f}'.format(event.ydata), 
                                xy=(event.xdata, event.ydata), xycoords='data', 
                                xytext=(1, 1), textcoords='offset points',
                                #xytext=(event.xdata, event.ydata), textcoords='data',
                                horizontalalignment="left", 
                                #arrowprops=dict(arrowstyle="simple", connectionstyle="arc3,rad=-0.2"),
                                bbox=dict(boxstyle="round", facecolor="w", edgecolor="0.5", alpha=0.9), 
                                fontsize='xx-small')
                        print('after annotate--' + str(time.thread_time()))
                        self.annotate_state[i].set_visible(True)
                        break
            
            print('before draw--' + str(time.thread_time()))
            #self.f.set_tight_layout(True)
            self.output_canvas.draw()
            print('after draw--' + str(time.thread_time()))

                #if( self.ax[0].contains(event)[0] == True):
            """if( self.isPointOnGraphLine(event.xdata, event.ydata, self.dfDaily, '4. close')):
                    if(self.annotate_state[0] != None):
                        self.annotate_state[0].set_visible(False)
                    self.annotate_state[0] = self.ax[0].annotate("Price: " + str(event.ydata), xy=(event.xdata, event.ydata),
                        xycoords='data', xytext=(event.xdata + 1, event.ydata + 1), textcoords='data',
                        horizontalalignment="left", arrowprops=dict(arrowstyle="simple", connectionstyle="arc3,rad=-0.2"),
                        bbox=dict(boxstyle="round", facecolor="w", edgecolor="0.5", alpha=0.9), fontsize='xx-small')
                        
                        #a = self.ax[0].annotate("Price: " + str(event.ydata), xy=(event.xdata, event.ydata),
                        #    xycoords='data', xytext=(event.x + 1, event.y), textcoords='offset points', arrowprops=dict(arrowstyle='-|>'))
                    self.annotate_state[0].set_visible(True)
                    self.output_canvas.draw()
                        #self.ax[0].draw()
                        #msgbx.showinfo('point', 'x=' + str(event.xdata) + '  y='+str(event.ydata))
                elif(self.annotate_state[0] != None):
                    self.annotate_state[0].set_visible(False)"""

            """elif (event.inaxes == self.ax[1]):
                msgbx.showinfo('If', 'ax[1]')
            elif (event.inaxes == self.ax[2]):
                msgbx.showinfo('If', 'ax[2]')
            elif (event.inaxes == self.ax[3]):
                msgbx.showinfo('If', 'ax[3]')
            elif (event.inaxes == self.ax[4]):
                msgbx.showinfo('If', 'ax[4]')
            elif (event.inaxes == self.ax[5]):
                msgbx.showinfo('If', 'ax[5]')
            elif (event.inaxes == self.ax[6]):
                msgbx.showinfo('If', 'ax[6]')
            elif (event.inaxes == self.ax[7]):
                msgbx.showinfo('If', 'ax[7]')
            elif (event.inaxes == self.ax[8]):
                msgbx.showinfo('If', 'ax[8]')"""
        #else:
        #    msgbx.showinfo('Else', 'Clicked ouside axes bounds but inside plot window')

    """ Method - OnRightClick
        This method will show popup menu if user right clicks a valid row"""
    def OnRightClick(self, event):
        try:
            if((self.output_tree.selectRowOnRightClick(event)==True) and (len(self.output_tree.is_market_holding_col_row())>0) and
                (self.output_tree.is_parent_item_selected() == False)):
                try:
                    self.popup_menu_righclick.grab_release()
                    bShouldRestMenu = self.TreeViewSelectionChanged()
                    if( bShouldRestMenu):
                        self.resetMenuticktoFalse()
                    self.popup_menu_righclick.grab_set()
                    self.popup_menu_righclick.tk_popup(event.x_root, event.y_root, 0)

                finally:
                    self.popup_menu_righclick.grab_release()
        except Exception as e:
            return

if __name__ == "__main__":
    portfolio_manager=PortfolioManager()




#v1.0
#v0.9 - All research graph via menu & mouse click
#v0.8 - Candlestick graphs
#v0.6
#v0.5
#v0.4
from tkinter import *
from tkinter import ttk
from tkinter import messagebox as msgbx

from alpha_vantage.timeseries import TimeSeries
from alpha_vantage.techindicators import TechIndicators
import pandas as pd
from pandas import DataFrame
from matplotlib.pyplot import Figure
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from matplotlib import interactive
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2Tk
import warnings
from datetime import date
from testdata import *
from mfdownloaddata import MFData

class ScriptTreeView(ttk.Treeview):
    def __init__(self, master=None, argTS = None, argTI = None, argFigure = None, argTestMode = None, argCanvas = None, argToolbar = None, argDataFolder = './scriptdata', **kw):
        super().__init__(master=master, **kw)
        
        self.bleftBtnReleased = False
        self.bleftDoubleClicked = False
        self.output_counter = 0
        self.ts = argTS
        self.ti = argTI
        self.f = argFigure
        self.graph_canvas = argCanvas
        self.graph_toolbar = argToolbar
        self.btestmode = argTestMode
        self.HOLDINGVAL = '_HOLDINGVAL_'
        self.datafolderpath = argDataFolder
        #self.script_tree = ttk.Treeview(master, selectmode='browse')
        #self.bind('<Double 1>', self.OnTreeDoubleClick)
        #self.bind('<Button 1>', self.OnTreeSingleClick)
        #self.bind('<ButtonRelease 1>', self.OnLeftBtnReleased)

        #scroll bar for Tree
        self.vert_scroll = ttk.Scrollbar(master, orient=VERTICAL, command=self.yview)
        self.configure(yscrollcommand=self.vert_scroll.set)
        self.horiz_scroll = ttk.Scrollbar(master, orient=HORIZONTAL, command=self.xview)
        self.configure(xscrollcommand=self.horiz_scroll.set)

    def OnLeftBtnReleased(self, event):
        self.bleftBtnReleased = True

    def OnTreeDoubleClick(self, event):
        self.bleftDoubleClicked = True

    def OnTreeSingleClick(self, event):
        self.bleftBtnReleased, self.bleftDoubleClicked = False, False
        self.after(300, self.callSingleDoubleClick, event)

    def callSingleDoubleClick(self, event):
        if self.bleftBtnReleased:
            if self.bleftDoubleClicked:
                self.TreeDoubleClick(event)
            else:
                self.TreeSingleClick(event)

    # tree view row double clicked
    def TreeDoubleClick(self, event):
        try:
            item=self.selection()[0]
            item2 = self.identify_row(event.y)
            if(item != item2):  #user double clicked on an item which is not currently selected
                return
        except IndexError:
            return
        script_name = self.item(item, "text")

        if self.btestmode:
            aapl_data = pd.read_csv("E:\\python_projects\\TestData\\daily_MSFT.csv")
        else:
            try:
                aapl_data, aapl_meta_data = self.ts.get_daily(symbol=script_name)
                # Not sure if we need the following line -- commenting for time being
                # aapl_sma is a df, aapl_meta_sma also a dict
                aapl_sma, aapl_meta_sma = self.ti.get_sma(symbol=script_name)
            except ValueError as error:
                msgbx.showerror("Alpha Vantage error", error)
                return

        # get users price & date
        dict_curr_row = self.item(script_name)
        purchase_price = dict_curr_row['values'][1]
        purchase_date =  dict_curr_row['values'][2]
        
        # Visualization
        f_temp=Figure(figsize=(15, 6), dpi=80, facecolor='w', edgecolor='k')
        plt.clf()
        if(self.btestmode):
            # aapl_data['close'].plot(title=script_name)
            plt.plot(aapl_data['close'], label='Stock price')
        else:
            
            #aapl_data['4. close'].plot(title=script_name)
            plt.plot(aapl_data['4. close'], label='Stock price')
            plt.plot(aapl_sma['SMA'], label='SMA')

        plt.title(script_name)
        plt.xlabel('Date')
        plt.ylabel('Price')
        if ((purchase_date != '') and (purchase_price != '')):
            plt.annotate('Your price point', (mdates.datestr2num(purchase_date), float(purchase_price)),
                    xytext=(15,15), textcoords='offset points', arrowprops=dict(arrowstyle='-|>'))
            plt.axhline(float(purchase_price), color='y') # will draw a horizontal line at purchase price
        plt.tight_layout()
        plt.legend(loc='upper left')
        plt.grid()
        plt.show()

    # single click handler for TreeView
    def TreeSingleClick(self, event):
        try:
            item=self.selection()[0]
            item2 = self.identify_row(event.y)
            if(item != item2):  #user clicked on an item which is not currently selected
                return
        except IndexError:
            return
        script_name = self.item(item, "text")
        # Get the data, returns a tuple
        # aapl_data is a pandas dataframe, aapl_meta_data is a dict
        if self.btestmode:
            aapl_data = pd.read_csv("E:\\python_projects\\TestData\\daily_MSFT.csv")
        else:
            try:
                aapl_data, aapl_meta_data = self.ts.get_daily(symbol=script_name)
            except ValueError as error:
                msgbx.showerror("Alpha Vantage error", error)
                return

        # Not sure if we need the following line -- commenting for time being
        # aapl_sma is a dict, aapl_meta_sma also a dict
        # aapl_sma, aapl_meta_sma = ti.get_sma(symbol=script_name)

        # Visualization
        if self.btestmode:
            self.f.clear()
            self.f.add_subplot(111, title=script_name, label='Daily close price', 
                xlabel='Date', ylabel='Closing price').plot(aapl_data['close'], label='Daily closing price')
        else:
            with warnings.catch_warnings(record=True) as w:
                warnings.simplefilter("always")
                self.f.clear()
                self.f.add_subplot(111, title=script_name, label='Daily close price', 
                    xlabel='Date', ylabel='Closing price').plot(aapl_data['4. close'], label='Daily closing price')
                # msgbx.showwarning("Plot waring", w)

        # f.suptitle(script_name)
        self.f.tight_layout()
        # f.set_label('Daily price seriese')
        self.f.legend(loc='upper right')

        self.graph_canvas.set_window_title(script_name)
        # toolbar=NavigationToolbar2Tk(output_canvas, output_canvas.get_tk_widget())
        self.graph_canvas.draw()
        self.graph_toolbar.update()

    def update_currentval(self, listmarketValues, listmodvalues, listrowValues, rowid, holdingiid):
        
        if(rowid == holdingiid):
            self.set(rowid, column=1, value=listmodvalues[0])
            self.set(rowid, column=2, value=listmodvalues[1])
            self.set(rowid, column=3, value=listmodvalues[2])
            self.set(rowid, column=4, value=listmodvalues[3])
            self.set(rowid, column=5, value=listmodvalues[4])

        status = ''
        closingprice = float(listmarketValues[4])
        currentvalue=0.00
        if( (rowid != holdingiid) and (len(listrowValues[2]) > 0) and (len(listrowValues[4]) > 0)):
            currentvalue = closingprice * float(listrowValues[2])
            if(currentvalue > float(listrowValues[4])):
                status = '↑'
            elif (currentvalue < float(listrowValues[4])):
                status = '↓'
            elif (currentvalue == float(listrowValues[4])):
                status = '↔'
        elif ( (rowid == holdingiid) and (len(listmodvalues[2]) > 0) and (len(listmodvalues[4]) > 0)):
            currentvalue = closingprice * float(listmodvalues[2])
            if(currentvalue > float(listmodvalues[4])):
                status = '↑'
            elif (currentvalue < float(listmodvalues[4])):
                status = '↓'
            elif (currentvalue == float(listmodvalues[4])):
                status = '↔'
        self.set(rowid, column=6, value=str(currentvalue))
        self.set(rowid, column=7, value=status)

    def print_values(self, argholdingiid, arg_heading_list, arg_values_list, arg_self_col_list, arg_self_val_list, counter):
        self.output_counter = counter
        if(self.output_counter>0):
            # now insert the data in tree
            iid_str = str(arg_values_list[0])
            if self.exists(iid_str) == True:
                #self.insert('', 'end', iid=iid_str, text=str(arg_values_list[0]), values=arg_values_list)
                #we have to find the last HOLDINGVAL iid
                childrows = self.get_children(iid_str)
                holdingctr = 1

                # first get the current closing price
                
                for child in childrows:
                        # now get  rows values only for self holding, we will not store market data
                    if(str(child).upper().find(self.HOLDINGVAL) >= 0):
                        split_list = str(child).split('_')
                        holdingctr = int(split_list[len(split_list)-1])
                        holdingctr +=1
                        row_val = self.item(child, 'values')
                        self.update_currentval(arg_values_list, arg_self_val_list, row_val, child, argholdingiid)

                if((argholdingiid == '') and (holdingctr >= 1) and (self.isValidHoldingRecord(arg_self_col_list, arg_self_val_list))):
                    #now insert the self data
                    curr_iid = iid_str + self.HOLDINGVAL + str(holdingctr)
                    idcol = self.insert(iid_str, "end", iid = curr_iid,text='')
                    for colid in range(len(arg_self_val_list)):
                        self.set(idcol, column=colid+1, value=arg_self_val_list[colid])
            else:   # update an existing item with new column values
                 #first enter the script name as the 0 column content
                self.insert('', 'end', iid=iid_str, text=str(arg_values_list[0]))

                #now insert the column for alpha
                idcol = self.insert(iid_str, "end", iid=iid_str+'_MARKETCOL' ,text='Market')
                for colid in range(len(arg_heading_list)):
                    self.set(idcol, column=colid+1, value=arg_heading_list[colid])

                #now insert the market data from alpha
                idcol = self.insert(iid_str, "end", iid= iid_str + '_MARKETVAL',text='')
                for colid in range(len(arg_values_list)):
                    self.set(idcol, column=colid+1, value=arg_values_list[colid])

                #now insert the column for self
                idcol = self.insert(iid_str, "end", iid = iid_str + '_HOLDINGCOL', text='Portfolio')
                for colid in range(len(arg_self_col_list)):
                    self.set(idcol, column=colid+1, value=arg_self_col_list[colid])

                #now insert the self data
                idcol = self.insert(iid_str, "end", iid = iid_str + self.HOLDINGVAL +'1',text='')
                for colid in range(len(arg_self_val_list)):
                    self.set(idcol, column=colid+1, value=arg_self_val_list[colid])

            self.focus(iid_str)
            self.selection_set(iid_str)
        self.output_counter += 1
        self.update()
        return self.output_counter

    # Method that creates columns in TreeView
    def print_heading(self, columnlen, counter):
        # get the column headings
        #global output_counter
        self.output_counter = counter
        if(self.output_counter==0):
            self.output_counter += 1
            self.column("#0", width=100, anchor='center')
            self.heading("#0", text='Script', anchor='center')
            self["columns"] = list(range(1, columnlen+1))
            for eachcol in range(1, columnlen+1):
                self.column(str(eachcol), width=100, anchor='center')
                self.heading(str(eachcol), text="", anchor='center')

        self.update()
        return self.output_counter


    """Method - get_stock_quote
        #*args = "Purchase Price=123.33", "Purchase Date=2019-10-10", "Purchase Qty=110", 
        #   "Commision=10", "Cost of Investment=1111"
        # Method to get current stock quote for given stock name"""
    def get_stock_quote(self, argHoldingIID = '', argStockName='', argPriceDf=DataFrame(), *args):
        #global bool_test
        dfstockname = DataFrame()
        listselfcol = list()
        listselfval = list()
        if(len(args) == 5):
            argctr = 0
            for eacharg in args:
                argsplit = eacharg.split('=')
                listselfcol.insert(argctr, argsplit[0].strip())
                listselfval.insert(argctr, argsplit[1].strip())
                argctr += 1
          
            try:
                listindex = listselfcol.index('Purchase Price')
                sPurchasePrice = listselfval[listindex]
                listindex = listselfcol.index('Purchase Date')
                sPurchaseDate = listselfval[listindex]
                listindex = listselfcol.index('Purchase Qty')
                sQty = listselfval[listindex]
                listindex = listselfcol.index('Commission Paid')
                sCommissionPaid = listselfval[listindex]
                listindex = listselfcol.index('Cost of Investment')
                sCost = listselfval[listindex]
            except ValueError as verr:
                msgbx.showerror("Error", "Insufficient arguments passed in *args")
                return
        else:
            msgbx.showerror("Error", "Insufficient arguments passed in *args")
            return
        
        try:
            if(argPriceDf.empty == True):
                if (self.btestmode==True):
                    testobj = PrepareTestData(argFolder=self.datafolderpath)
                    dfstockname = testobj.GetQuoteEndPoint(argStockName)
                else:
                    dfstockname, meta_data = self.ts.get_quote_endpoint(argStockName)
            else:
                dfstockname = argPriceDf
        except ValueError as error:
            msgbx.showerror("Alpha Vantage Error", error)
            return
        currclosingprice = float(dfstockname.values[0][4])
        status = ''
        currentvalue=0.00
        if((len(sQty) > 0) and (len(sCost) > 0)):
            currentvalue = currclosingprice * float(sQty)
            if(currentvalue > float(sCost)):
                status = '↑'
            elif (currentvalue < float(sCost)):
                status = '↓'
            elif (currentvalue == float(sCost)):
                status = '↔'
            
        listselfcol.insert(argctr, 'Current Value')
        listselfval.insert(argctr, str(currentvalue))
        argctr += 1
        listselfcol.insert(argctr, 'Status')
        listselfval.insert(argctr, status)  #alt 24 ↑, alt 25 ↓, alt 29 ↔

        dfcolumnlen = len(dfstockname.columns)
        heading_list=list(dfstockname.columns[0:dfcolumnlen])

        if(dfcolumnlen > len(listselfcol)):
            self.output_counter = self.print_heading(dfcolumnlen, self.output_counter)
        else:
            self.output_counter = self.print_heading(len(listselfcol), self.output_counter)
        
        values_list=list((dfstockname.values[0:dfcolumnlen])[0])
        #commenting for heirarchy
        #self.print_values(values_list)
        self.output_counter = self.print_values(argHoldingIID, heading_list, values_list, listselfcol, listselfval, self.output_counter)


    """method - get_parent_item
        returns the parent item id of the selected item.
        if the selected item is child then will get its parent iid and return the same
        if the selected item is parent then it will return the same
        if no item is selected it will return ''  """
    def get_parent_item(self, argitem = None):
        try:
            if(argitem == None):
                item = self.selection()[0]
            else:
                item = argitem
            parentitem = self.parent(item)
        except IndexError:
            return ''
        script_name = ''
        if(len(parentitem) <= 0):
            script_name = self.item(item, "text")
        else:
            script_name = self.item(parentitem, "text")
        return script_name

    def is_parent_item_selected(self, argitem = None):
        try:
            if(argitem == None):
                item = self.selection()[0]
            else:
                item = argitem
            #if a parent HDFC.BSE is already selected then .parent will return ''
            parentitem = self.parent(item)
        except IndexError:
            return False
        script_name = ''
        if(len(parentitem) > 0):
            #script_name = self.item(item, "text")
            return False
        #else:
        #    script_name = self.item(parentitem, "text")
        #return script_name
        return True

    """ method - is_market_holding_col_row
        given item or for selected item, 
            the method will return '' if iid has MARKETCOL or HOLDINGCOL in it
            else return the iid string
            in case of exception will retun ''    """
    def is_market_holding_col_row(self, argitem = None):
        try:
            if(argitem == None):
                item = self.selection()[0]
            else:
                item = argitem
            #script_name = self.item(item, "text")

            if((item.upper().find("MARKETCOL") >= 0) or 
                (item.upper().find("HOLDINGCOL") >= 0) or
                (item.upper().find("MARKETVAL") >= 0)):
                return ''
            else:
                return item
        except IndexError:
            return ''

    """ method - isValidHoldingRecord
        will return True if Purchase Price, Purchase Date, Quantity are not empty
        else returns False """
    def isValidHoldingRecord(self,arg_self_col_list, arg_self_val_list):
            listindex = arg_self_col_list.index('Purchase Price')
            sPurchasePrice = arg_self_val_list[listindex]
            listindex = arg_self_col_list.index('Purchase Date')
            sPurchaseDate = arg_self_val_list[listindex]
            listindex = arg_self_col_list.index('Purchase Qty')
            sQty = arg_self_val_list[listindex]
            if((len(sPurchasePrice) > 0) and (len(sPurchaseDate)>0) and (len(sQty)>0)):
                return True

            return False

    """ Method - selectRowOnRightClick
        will find the row where user right clicked
        if currently selected row & right clicked row are not the same, it will select the 
            right clicked row"""
    def selectRowOnRightClick(self, event=None):
            try:
                item = self.identify_row(event.y)
                item2 =self.selection()[0]
                if (item == ''): #clicked where no row exists
                    return False
                elif (item != item2):
                    self.selection_set(item)
            except Exception as e:
                return False
            return True

    def get_mf_quote(self, argHoldingIID = '', argStockName='', argPriceDf=DataFrame(), *args):
        #global bool_test
        dfstockname = DataFrame()
        listselfcol = list()
        listselfval = list()
        if(len(args) == 5):
            argctr = 0
            for eacharg in args:
                argsplit = eacharg.split('=')
                listselfcol.insert(argctr, argsplit[0].strip())
                listselfval.insert(argctr, argsplit[1].strip())
                argctr += 1
          
            try:
                listindex = listselfcol.index('Purchase Price')
                sPurchasePrice = listselfval[listindex]
                listindex = listselfcol.index('Purchase Date')
                sPurchaseDate = listselfval[listindex]
                listindex = listselfcol.index('Purchase Qty')
                sQty = listselfval[listindex]
                listindex = listselfcol.index('Commission Paid')
                sCommissionPaid = listselfval[listindex]
                listindex = listselfcol.index('Cost of Investment')
                sCost = listselfval[listindex]
            except ValueError as verr:
                msgbx.showerror("Error", "Insufficient arguments passed in *args")
                return
        else:
            msgbx.showerror("Error", "Insufficient arguments passed in *args")
            return
        
        try:
            if(argPriceDf.empty == True):
                obj = MFData()
                self.currentNavDF = obj.DownloadCurrentMF()
                dfstockname = self.currentNavDF[self.currentNavDF['Scheme Name']==argStockName]
            else:
                dfstockname = argPriceDf
        except Exception as error:
            msgbx.showerror("Error whiel getting current NAV", error)
            return
        currNav = float(dfstockname.values[0][4])
        status = ''
        currentvalue=0.00
        if((len(sQty) > 0) and (len(sCost) > 0)):
            currentvalue = currNav * float(sQty)
            if(currentvalue > float(sCost)):
                status = '↑'
            elif (currentvalue < float(sCost)):
                status = '↓'
            elif (currentvalue == float(sCost)):
                status = '↔'
            
        listselfcol.insert(argctr, 'Current Value')
        listselfval.insert(argctr, str(currentvalue))
        argctr += 1
        listselfcol.insert(argctr, 'Status')
        listselfval.insert(argctr, status)  #alt 24 ↑, alt 25 ↓, alt 29 ↔

        dfcolumnlen = len(dfstockname.columns)
        heading_list=list(dfstockname.columns[0:dfcolumnlen])

        if(dfcolumnlen > len(listselfcol)):
            self.output_counter = self.print_heading(dfcolumnlen, self.output_counter)
        else:
            self.output_counter = self.print_heading(len(listselfcol), self.output_counter)
        
        values_list=list((dfstockname.values[0:dfcolumnlen])[0])
        #commenting for heirarchy
        #self.print_values(values_list)
        self.output_counter = self.print_values(argHoldingIID, heading_list, values_list, listselfcol, listselfval, self.output_counter)
